SystemOrganization addCategory: #'Phantom-Tests'!

TestCase subclass: #AspectPrecedenceRulesParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

!AspectPrecedenceRulesParserTest methodsFor: 'parse' stamp: 'DanielGaldames 3/29/2011 22:09'!
testAspectParser
	| parser |
	parser := PhPrecedenceRulesParser new.
	self assert: (parser matches: 'AsdfAspect').
	self assert: (parser matches: '*').
	self deny: (parser matches: '*+').
	self deny: (parser matches: '+*').
	self deny: (parser matches: '+').
	self deny: (parser matches: '1Aspect').
	self deny: (parser matches: 'Asp ect').! !

!AspectPrecedenceRulesParserTest methodsFor: 'parse' stamp: 'DanielGaldames 3/24/2011 00:19'!
testAspectPlusParser
	| parser |
	parser := PhPrecedenceRulesParser new.
	self assert: (parser matches: 'AsdfAspect+').
	self deny: (parser matches: '+Aspect').
	self deny: (parser matches: 'Foo+Aspect').
	self deny: (parser matches: '+aspect').
	self deny: (parser matches: '+').! !

!AspectPrecedenceRulesParserTest methodsFor: 'match' stamp: 'DanielGaldames 3/25/2011 12:14'!
testAspectPlusPrecedenceMatch
	| parser target  set classes |
	parser := PhPrecedenceRulesParser new.
	set := parser parse: 'PhMockAspectFoo+'.
	classes := PhMockAspectFoo allSubclasses.
	target := Set new.
	target add: (AspectNode new aspect: PhMockAspectFoo).
	classes do: [:each | target add: (AspectNode new aspect: each)].
	self assert: (set = target)! !

!AspectPrecedenceRulesParserTest methodsFor: 'parse' stamp: 'DanielGaldames 3/24/2011 00:23'!
testAspectStarParser
	| parser |
	parser := PhPrecedenceRulesParser new.
	self assert: (parser matches: 'AsdfAspect*').
	self deny: (parser matches: 'Asp*ect')..
	self deny: (parser matches: 'Aspect *')..

! !

!AspectPrecedenceRulesParserTest methodsFor: 'parse' stamp: 'DanielGaldames 3/24/2011 00:24'!
testAspectStarPlusParser
	| parser |
	parser := PhPrecedenceRulesParser new.
	self assert: (parser matches: 'AsdfAspect*+').
	self deny: (parser matches: 'Aspect+*')..
	self deny: (parser matches: 'Aspect *+')..! !

!AspectPrecedenceRulesParserTest methodsFor: 'match' stamp: 'DanielGaldames 3/25/2011 12:14'!
testAspectStarPrecedenceMatch
	| parser set target classes regex |
	regex := 'PhAspectTest.*' asRegex.
	parser := PhPrecedenceRulesParser new.
	set := parser parse: 'PhAspectTest*'.
	classes := PhAspect allSubclasses select: [:class | regex matches: class name asString].
	target := Set new.
	classes do: [:class | target add: (AspectNode new aspect: class)].
	self assert: (set = target).! !

!AspectPrecedenceRulesParserTest methodsFor: 'parse' stamp: 'DanielGaldames 3/24/2011 00:24'!
testStarAspectParser
	| parser |
	parser := PhPrecedenceRulesParser new.
	self assert: (parser matches: '*AsdfAspect').
	self deny: (parser matches: 'Asp*ect')..
	self deny: (parser matches: '* Aspect')..! !

!AspectPrecedenceRulesParserTest methodsFor: 'parse' stamp: 'DanielGaldames 3/24/2011 00:25'!
testStarAspectPlusParser
	| parser |
	parser := PhPrecedenceRulesParser new.
	self assert: (parser matches: '*AsdfAspect+').
	self deny: (parser matches: '*+Aspect')..
	self deny: (parser matches: '* Aspect +')..! !

!AspectPrecedenceRulesParserTest methodsFor: 'match' stamp: 'DanielGaldames 3/25/2011 12:15'!
testStarAspectPrecedenceMatch
	| parser set target classes regex |
	regex := '\w*MockAspect1' asRegex.
	parser := PhPrecedenceRulesParser new.
	set := parser parse: '*MockAspect1'.
	classes := PhAspect allSubclasses select: [:class | regex matches: class name asString].
	target := Set new.
	classes do: [:class | target add: (AspectNode new aspect: class)].
	self assert: (set = target).! !

!AspectPrecedenceRulesParserTest methodsFor: 'parse' stamp: 'DanielGaldames 3/24/2011 00:27'!
testStarAspectStarParser
	| parser |
	parser := PhPrecedenceRulesParser new.
	self assert: (parser matches: '*AsdfAspect*').
	self deny: (parser matches: '* Aspect *')..
	self deny: (parser matches: '*Aspect *')..
	self deny: (parser matches: '* Aspect*')..! !

!AspectPrecedenceRulesParserTest methodsFor: 'parse' stamp: 'DanielGaldames 3/24/2011 00:31'!
testStarAspectStarPlusParser
	| parser |
	parser := PhPrecedenceRulesParser new.
	self assert: (parser matches: '*AsdfAspect*+').
	self deny: (parser matches: '*AsdfAspect+*').
	self deny: (parser matches: '*+AsdfAspect*+').
	self deny: (parser matches: '+*AsdfAspect*+').
	self deny: (parser matches: '* Aspect *')..
	self deny: (parser matches: '*Aspect *')..
	self deny: (parser matches: '* Aspect*')..! !

!AspectPrecedenceRulesParserTest methodsFor: 'match' stamp: 'DanielGaldames 3/25/2011 12:13'!
testStarAspectStarPrecedenceMatch
	| parser set target classes regex |
	regex := '[\w\d]*MockAspect[\w\d]*' asRegex.
	parser := PhPrecedenceRulesParser new.
	set := parser parse: '*MockAspect*'.
	classes := PhAspect allSubclasses select: [:class | regex matches: class name asString].
	target := Set new.
	classes do: [:class | target add: (AspectNode new aspect: class)].
	self assert: (set = target).! !

TestCase subclass: #AspectPrecedenceRulesProcessorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

!AspectPrecedenceRulesProcessorTest methodsFor: 'graph-generation' stamp: 'DanielGaldames 3/25/2011 13:57'!
testGraphGenerationLongCollection
	| collection processor graph n1 n2 n3 n4 n5 n6 |
	processor := PhPrecedenceRulesProcessor new.
	collection := OrderedCollection new.
	n1 := AspectNode new aspect: PhMockAspect1.
	n2 := AspectNode new aspect: PhMockAspect2.
	n3 := AspectNode new aspect: PhMockAspect3.
	n4 := AspectNode new aspect: PhMockAspect4.
	n5 := AspectNode new aspect: PhMockAspect5.
	n6 := AspectNode new aspect: PhMockAspect6.
	collection add: (Set with: n1).
	collection add: (Set with: n2 with: n3).
	collection add: (Set with: n4 with: n5).
	collection add: (Set with: n6).
	graph := processor getGraphFrom: collection.
	self assert: (graph nodeSet size = 6).
	self assert: (graph nodeSet includes: n1).
	self assert: (graph nodeSet includes: n2).
	self assert: (graph nodeSet includes: n3).
	self assert: (graph nodeSet includes: n4).
	self assert: (graph nodeSet includes: n5).
	self assert: (graph nodeSet includes: n6).
	self assert: ((graph nodeSet like: n1) subnodes includes: n2).
	self assert: ((graph nodeSet like: n1) subnodes includes: n3).
	self assert: ((graph nodeSet like: n2) subnodes includes: n4).
	self assert: ((graph nodeSet like: n2) subnodes includes: n5).
	self assert: ((graph nodeSet like: n3) subnodes includes: n4).
	self assert: ((graph nodeSet like: n3) subnodes includes: n5).
	self assert: ((graph nodeSet like: n4) subnodes includes: n6).
	self assert: ((graph nodeSet like: n5) subnodes includes: n6).
	self assert: ((graph nodeSet like: n1) subnodes size = 2).
	self assert: ((graph nodeSet like: n2) subnodes size = 2).
	self assert: ((graph nodeSet like: n3) subnodes size = 2).
	self assert: ((graph nodeSet like: n4) subnodes size = 1).
	self assert: ((graph nodeSet like: n5) subnodes size = 1).
	self assert: ((graph nodeSet like: n6) subnodes size = 0).! !

!AspectPrecedenceRulesProcessorTest methodsFor: 'graph-generation' stamp: 'DanielGaldames 3/25/2011 13:59'!
testGraphGenerationOneNodeMultiNodes
	| collection processor graph n1 n2 n3 n4 |
	processor := PhPrecedenceRulesProcessor new.
	collection := OrderedCollection new.
	n1 := AspectNode new aspect: PhMockAspect1.
	n2 := AspectNode new aspect: PhMockAspect2.
	n3 := AspectNode new aspect: PhMockAspect3.
	n4 := AspectNode new aspect: PhMockAspect4.
	collection add: (Set with: n1).
	collection add: (Set with: n2 with: n3 with: n4).
	graph := processor getGraphFrom: collection.
	self assert: (graph nodeSet size = 4).
	self assert: (graph nodeSet includes: n1).
	self assert: (graph nodeSet includes: n2).
	self assert: (graph nodeSet includes: n3).
	self assert: (graph nodeSet includes: n4).
	self assert: ((graph nodeSet like: n1) subnodes includes: n2).
	self assert: ((graph nodeSet like: n1) subnodes includes: n3).
	self assert: ((graph nodeSet like: n1) subnodes includes: n4).
	self assert: ((graph nodeSet like: n1) subnodes size = 3).! !

!AspectPrecedenceRulesProcessorTest methodsFor: 'graph-generation' stamp: 'DanielGaldames 3/24/2011 21:44'!
testGraphGenerationTwoNodes
	| collection processor graph n1 n2 |
	processor := PhPrecedenceRulesProcessor new.
	collection := OrderedCollection new.
	n1 := AspectNode new aspect: PhMockAspect1.
	n2 := AspectNode new aspect: PhMockAspect2.
	collection add: (Set with: n1).
	collection add: (Set with: n2).
	graph := processor getGraphFrom: collection.
	self assert: (graph nodeSet size = 2).
	self assert: (graph nodeSet includes: n1).
	self assert: (graph nodeSet includes: n2).
	self assert: ((graph nodeSet like: n1) subnodes includes: n2).  ! !

!AspectPrecedenceRulesProcessorTest methodsFor: 'graph-generation' stamp: 'DanielGaldames 3/24/2011 22:19'!
testGraphMerge
	| graph1 graph2 n1g1 n2 n1g2 n3 |
	n1g1 := AspectNode new aspect: PhMockAspect1.
	n2 := AspectNode new aspect: PhMockAspect2.
	n1g2 := AspectNode new aspect: PhMockAspect1.
	n3 := AspectNode new aspect: PhMockAspect3.
	n1g1 addNode: n2.
	n1g2 addNode: n3.
	graph1 := AspectGraph new.
	graph1 addAll: (Array with: n1g1 with: n2).
	graph2 := AspectGraph new.
	graph2 addAll: (Array with: n1g2 with: n3).
	graph1 merge: graph2.
	self assert: ((graph1 nodeSet like: n1g1) subnodes includes: n2).
	self assert: ((graph1 nodeSet like: n1g1) subnodes includes: n3).
	self assert: ((graph1 nodeSet like: n1g2) subnodes includes: n2).
	self assert: ((graph1 nodeSet like: n1g2) subnodes includes: n3).     
	self assert: ((graph1 nodeSet like: n1g1) subnodes size = 2).! !

TestCase subclass: #PhAspectTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

!PhAspectTests methodsFor: 'add-remove' stamp: 'dgg 10/7/2011 17:13'!
testAddAdvice
| pc adv asp |
	pc := PhPointcut new
		receivers: 'PhMockClass'; 
		selectors: 'test'.
	adv := PhAdvice new
		advice: [:ctx | 'around'];
		pointcut: pc;
		type: #around.
	asp := PhAspect new.
	asp add: adv.
	self assert: (asp adviceCollection includes: adv).
	asp remove: adv.
	self deny: (asp adviceCollection includes: adv)
	! !

!PhAspectTests methodsFor: 'add-remove' stamp: 'dgg 9/29/2011 15:52'!
testAddClassModifier
| pc mod asp |
	pc := PhPointcut new
		receivers: 'PhMockClass'; 
		selectors: 'test'.
	mod := PhClassModifier new on: (PhPointcut receivers: 'PhMockClass' selectors: #any asParser).
	asp := PhAspect new.
	asp addClassModifier: mod.
	self assert: (asp classModifiers includes: mod).
	asp removeClassModifier: mod.
	self deny: (asp classModifiers includes: mod)
	! !

!PhAspectTests methodsFor: 'as yet unclassified' stamp: 'dgg 10/7/2011 17:14'!
testAdviceAsBlock
	| mock  pc adv asp |
	mock := PhMockClass new.
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'foo'.
	adv := PhAdvice new
		advice: [:ctx | 'test'];
		pointcut: pc;  
		type: #around.
	asp := PhAspect new.
	asp add: adv.
	[asp install.
	self assert: (mock foo = 'test')] ensure: [asp uninstall]! !

!PhAspectTests methodsFor: 'advice-manipulation' stamp: 'dgg 5/2/2011 15:26'!
testAdviceRemovalFromAfter
	| asp1 asp2 pc mock |
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'counter';
		context: #(#advice #receiver).
		
	asp1 := PhAspect new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context |
				context afterAdvice: #()];
			type: #before).
	asp2 := PhAspect new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | context receiver plus];
		type: #after).
	
	[asp2 install.
	mock := PhMockClass new.
	self assert: (mock counter = 0).
	self assert: (mock counter = 1).
	asp1 install.
	"asp1 will remove plus advice on counter"
	self assert: (mock counter = 2).
	self assert: (mock counter = 2)] ensure: [
	asp1 uninstall.
	asp2 uninstall.]! !

!PhAspectTests methodsFor: 'advice-manipulation' stamp: 'dgg 5/2/2011 15:29'!
testAdviceRemovalFromAround
	| asp1 asp2 pc mock |
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'counter';
		context: #(#advice #receiver).
		
	asp1 := PhAspect new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context |
				context aroundAdvice: #()];
			type: #before).
	asp2 := PhAspect new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | 'foo'];
		type: #around).
	[asp2 install. 
	mock := PhMockClass new.
	self assert: (mock counter = 'foo').
	asp1 install.
	"asp1 will remove plus advice on counter"
	self assert: (mock counter = 0)] ensure: [
	asp1 uninstall.
	asp2 uninstall.]! !

!PhAspectTests methodsFor: 'advice-manipulation' stamp: 'dgg 5/4/2011 16:21'!
testAdviceRemovalFromBefore
	| asp1 asp2 pc mock |
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'counter';
		context: #(#advice #receiver #proceed).
		
	asp1 := PhAspect new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context |
				context aroundAdvice: (context aroundAdvice allButFirst)];
			type: #before).
	asp2 := PhAspect new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | context receiver plus. context proceed];
		type: #around);
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | context proceed];
		type: #around).
	[asp2 install.
	mock := PhMockClass new.
	self assert: (mock counter = 1).
	asp1 install.
	"asp1 will remove plus advice on counter"
	self assert: (mock counter = 1).
	self assert: (mock counter = 1)] ensure: [
	asp1 uninstall.
	asp2 uninstall.]! !

!PhAspectTests methodsFor: 'as yet unclassified' stamp: 'dgg 10/7/2011 17:15'!
testAdviceSendTo
	| mock  pc adv asp |
	mock := PhMockClass new.
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'foo'.
	adv := PhAdvice new
		send: #adv: to: mock;
		pointcut: pc;  
		type: #around.
	asp := PhAspect new.
	asp add: adv.
	[asp install.
	self assert: (mock foo = 'test')] ensure: [asp uninstall]! !

!PhAspectTests methodsFor: 'as yet unclassified' stamp: 'dgg 5/2/2011 15:31'!
testAfterAdvice
	|pc test advAfter aspect mock |
	mock := PhMockClass new.
	self assert: (mock test = 'test').
	aspect := PhAspect new.
	pc := (PhPointcut new) receivers: 'PhMockClass'; selectors: 'test'; context: #(#receiver).
	advAfter := (PhAdvice new) advice: [:context | context receiver instVarNamed: 'testString' put: 'afterTest' ]; pointcut: pc; type: #after.  
	aspect add: advAfter.
	
	[aspect install.

	self assert: (mock test = 'test').
	self assert: (mock testString = 'afterTest')] ensure: [aspect uninstall]  ! !

!PhAspectTests methodsFor: 'as yet unclassified' stamp: 'dgg 10/7/2011 17:16'!
testAroundAdvice
	| pc adv asp mock |
	mock := PhMockClass new.
	pc := PhPointcut new
		receivers: 'PhMockClass'; 
		selectors: 'test'.
	adv := PhAdvice new
		advice: [:ctx | 'around'];
		pointcut: pc;
		type: #around.
	asp := PhAspect new.
	asp add: adv.
	
	[asp install.
	
	self assert: (mock test = 'around')] ensure: [asp uninstall]! !

!PhAspectTests methodsFor: 'as yet unclassified' stamp: 'dgg 11/28/2011 20:27'!
testAroundAround
	| pc advice innerAspect outerAspect mock |
	mock := PhMockClass new.
	pc := PhPointcut receivers: 'PhMockClass' selectors: 'method2' context: #(#receiver #proceed).
	advice := PhAdvice around: pc advice: [ :context | 'aspectualBehavior' ].
	innerAspect := PhAspect new add: advice.
	pc := PhPointcut receivers: 'PhMockClass' selectors: 'callMethod2' context: #(#receiver #proceed).
	advice := PhAdvice around: pc advice: [ :context | context proceed ].
	outerAspect := PhAspect new add: advice.
	[ 
	innerAspect install.
	outerAspect install.
	self assert: mock callMethod2 = 'aspectualBehavior' ]
		ensure: [ 
			innerAspect uninstall.
			outerAspect uninstall ]! !

!PhAspectTests methodsFor: 'as yet unclassified' stamp: 'dgg 5/2/2011 15:32'!
testAroundProceedAdvice
	| pc adv asp mock |
	mock := PhMockClass new.
	pc := PhPointcut new
		receivers: 'PhMockClass'; 
		selectors: 'test';
		context: #(#proceed).  
	adv := PhAdvice new
		advice: [:context | 'around', context proceed];
		pointcut: pc;
		type: #around.
	asp := PhAspect new.
	asp add: adv.
	
	[asp install.
	
	self assert: (mock test = 'aroundtest')] ensure: [asp uninstall]! !

!PhAspectTests methodsFor: 'as yet unclassified' stamp: 'dgg 5/2/2011 15:32'!
testAroundProceedWithArgsAdvice
	| pc adv asp mock |
	mock := PhMockClass new.
	pc := PhPointcut new
		receivers: 'PhMockClass'; 
		selectors: 'echo:';
		context: #(#proceed #arguments).
	adv := PhAdvice new
		advice: [:context |  context proceed: (context arguments collect: [:a | a asUppercase])];
		pointcut: pc;
		type: #around.
	asp := PhAspect new.
	asp add: adv.
	
	[asp install.
	
	self assert: ((mock echo: 'test') = 'TEST')] ensure: [asp uninstall]! !

!PhAspectTests methodsFor: 'as yet unclassified' stamp: 'dgg 5/2/2011 15:33'!
testBeforeAdvice
	|pc advBefore aspect |
	self assert: ((PhMockClass new) test = 'test').
	aspect := PhAspect new.
	pc := (PhPointcut new) receivers: 'PhMockClass'; selectors: 'test'; context: #(#receiver).
	advBefore := (PhAdvice new) advice: [:context | context receiver instVarNamed: 'testString' put: 'beforeTest' ]; pointcut: pc; type: #before.  
	aspect add: advBefore.
	
	[aspect install.
	self assert: ((PhMockClass new) test = 'beforeTest')] ensure: [aspect uninstall].  ! !

!PhAspectTests methodsFor: 'as yet unclassified' stamp: 'dgg 5/2/2011 15:24'!
testBeforeAfterAdvice
	|pc beforeAdv advAfter aspect mock |

	mock := PhMockClass new.
	self assert: (mock test = 'test').
	aspect := PhAspect new.
	pc := (PhPointcut new) receivers: 'PhMockClass'; selectors: 'test'; context: #(#receiver).
	beforeAdv := (PhAdvice new) advice: [:context | context receiver instVarNamed: 'testString' put: ('before',(context receiver testString )) ]; pointcut: pc; type: #before.  
	advAfter := (PhAdvice new) advice: [:context | context receiver instVarNamed: 'testString' put: ((context receiver testString),'after') ]; pointcut: pc; type: #after.  
	aspect add: advAfter.
	aspect add: beforeAdv.
	[aspect install.
	self assert: (mock test = 'beforetest').  self assert: (mock testString = 'beforetestafter').] ensure: [aspect uninstall. ].
  	
	! !

!PhAspectTests methodsFor: 'as yet unclassified' stamp: 'dgg 11/28/2011 20:28'!
testCallAround
	| pc advice innerAspect outerAspect mock |
	mock := PhMockClass new.
	pc := PhPointcut receivers: 'PhMockClass' selectors: 'method2' context: #(#receiver #proceed).
	advice := PhAdvice around: pc advice: [ :context | 'aspectualBehavior' ].
	innerAspect := PhAspect new add: advice.
	pc := PhPointcut receivers: 'PhMockClass' selectors: 'callMethod2' context: #(#receiver).
	advice := PhAdvice after: pc advice: [ :context | ].
	outerAspect := PhAspect new add: advice.
	[ 
	innerAspect install.
	outerAspect install.
	self assert: mock callMethod2 = 'aspectualBehavior' ]
		ensure: [ 
			innerAspect uninstall.
			outerAspect uninstall ]! !

!PhAspectTests methodsFor: 'chained-proceed' stamp: 'dgg 5/4/2011 02:18'!
testChainedProceed1
	|  asp pc |
	asp := PhAspect new.
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'test';
		context: #(#proceed).
	asp add: (PhAdvice new
		advice: [:context | 'test',context proceed];
		pointcut: pc;
		type: #around).
	asp add: (PhAdvice new
		advice: [:context | 'test',context proceed];
		pointcut: pc;
		type: #around).
	asp add: (PhAdvice new
		advice: [:context | 'test',context proceed];
		pointcut: pc;
		type: #around).
	
	[asp install.
	self assert: (PhMockClass new test = 'testtesttesttest')] ensure: [asp uninstall.]! !

!PhAspectTests methodsFor: 'chained-proceed' stamp: 'dgg 2/20/2012 20:01'!
testChainedProceed2
	|  asp pc |
	asp := PhAspect new.
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'test';
		context: #(#proceed).
	asp add: (PhAdvice new
		advice: [:context | 'bar'];
		pointcut: pc;
		type: #around).
	asp add: (PhAdvice new
		advice: [:context | 'foo',context proceed];
		pointcut: pc;
		type: #around).
	asp add: (PhAdvice new
		advice: [:context | 'test',context proceed];
		pointcut: pc;
		type: #around).
	
	[asp install.
	PhMockClass new test.
	self assert: (PhMockClass new test = 'bar')] ensure: [asp uninstall.]! !

!PhAspectTests methodsFor: 'chained-proceed' stamp: 'dgg 2/20/2012 20:01'!
testChainedProceed3
	|  asp pc |
	asp := PhAspect new.
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'test';
		context: #(#proceed).
	asp add: (PhAdvice new
		advice: [:context | context proceed];
		pointcut: pc;
		type: #around).
	asp add: (PhAdvice new
		advice: [:context | context proceed];
		pointcut: pc;
		type: #around).
	asp add: (PhAdvice new
		advice: [:context | 'bar',context proceed];
		pointcut: pc;
		type: #around).
	
	[asp install.
	self assert: (PhMockClass new test = 'bartest')] ensure: [asp uninstall.]! !

!PhAspectTests methodsFor: 'chained-proceed' stamp: 'dgg 2/20/2012 20:02'!
testChainedProceed4
	|  asp pc |
	asp := PhAspect new.
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'test';
		context: #(#proceed).
	asp add: (PhAdvice new
		advice: [:context | 'bar',context proceed];
		pointcut: pc;
		type: #around).
	
	[asp install.
	self assert: (PhMockClass new test = 'bartest')] ensure: [asp uninstall.]! !

!PhAspectTests methodsFor: 'chained-proceed' stamp: 'dgg 2/20/2012 20:03'!
testChainedProceed5
	|  asp pc |
	asp := PhAspect new.
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'test';
		context: #(#proceed).
	asp add: (PhAdvice new
		advice: [:context | 'asdf'. context proceed];
		pointcut: pc;
		type: #around).
	asp add: (PhAdvice new
		advice: [:context | 'foo',context proceed];
		pointcut: pc;
		type: #around).
	asp add: (PhAdvice new
		advice: [:context | 'bar',context proceed];
		pointcut: pc;
		type: #around).
	
	[asp install.
	self assert: (PhMockClass new test = 'foobartest')] ensure: [asp uninstall.]! !

!PhAspectTests methodsFor: 'advice-manipulation' stamp: 'dgg 2/20/2012 20:04'!
testChangeCurrent
	|pc asp1 asp2 asp3 |
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'foo';
		context: #(#advice #proceed).
		
	asp1 := PhMockAspect1 new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context |
				| current | 
				current := context currentAdvice asOrderedCollection select: [:each | each owner class = PhMockAspect2].
				context currentAdvice: current continueAt: 1.
				context proceed];
			type: #around).
	asp2 := PhMockAspect2 new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | 'ok'];
			type: #around).
	asp3 := PhMockAspect3 new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | 'not', context proceed asString];
			type: #around);
		precedence: #('PhMockAspect1' 'PhMockAspect3' 'PhMockAspect2').
	
	[asp1 install.
	asp2 install.
	asp3 install.
	self assert: (PhMockClass new foo = 'ok')] ensure: [asp1 uninstall. asp2 uninstall. asp3 uninstall.]! !

!PhAspectTests methodsFor: 'advice-manipulation' stamp: 'dgg 2/20/2012 20:06'!
testChangeOrder
	|pc asp |
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'foo';
		context: #(#advice).
		
	asp := PhAspect new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | 
				context aroundAdvice: (context aroundAdvice asArray reversed).
				context proceed.];
			type: #before);
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | 'bar'];
			type: #around);
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | 'foo'];
			type: #around).
	[asp install.
	self assert: (PhMockClass new foo = 'foo')] ensure: [asp uninstall.]! !

!PhAspectTests methodsFor: 'advice-manipulation' stamp: 'dgg 2/20/2012 19:57'!
testCurrentAdviceRemove
	| asp1 asp2 asp3 pc |
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'testString';
		precedence: #('PhMockAspect1' 'PhMockAspect2' 'PhMockAspect3');
		context: #(#advice).
		
	asp1 := PhMockAspect1 new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context |
				| advices |
				advices := (context currentAdvice select: [:each | (each owner class = PhMockAspect2) not]).
				context currentAdvice: advices continueAt: (context currentAdviceIndex +1).
				context proceed];
			type: #around)..
	asp2 := PhMockAspect2 new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context |
				'foo'];
			type: #around).
	asp3 := PhMockAspect3 new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | 
				'bar'.];
			type: #around).
	
	  
	[asp2 install.
	asp3 install.
	self assert: (PhMockClass new testString = 'foo').
	asp1 install.
	self assert: (PhMockClass new testString = 'bar')
	] ensure: [
	asp1 uninstall.
	asp2 uninstall.
	asp3 uninstall]! !

!PhAspectTests methodsFor: 'inherited-precedence' stamp: 'dgg 5/2/2011 15:35'!
testInheritedPrecedence
	| asp1 asp2 asp3 asp4 asp5 asp6 pc |
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'test';
		context: #(#receiver).
	asp1 := PhAspect new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | context receiver test: 'asp1'];
			type: #before).
	asp2 := PhMockAspect1 new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | context receiver test: (context receiver testString), 'asp2'];
			type: #before).
	asp3 := PhMockAspectFoo1 new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | context receiver test: (context receiver testString), 'asp3'];
			type: #before).
	asp4 := PhMockFooAspect4 new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | context receiver test: (context receiver testString), 'asp4'];
			type: #before).
			
	  
	[
	asp4 install.
	asp3 install.
	asp2 install.
	asp1 install.
	self assert: (PhMockClass new test = 'asp1asp2asp3asp4')] ensure: [
	asp1 uninstall.
	asp2 uninstall.
	asp3 uninstall.
	asp4 uninstall.]! !

!PhAspectTests methodsFor: 'as yet unclassified' stamp: 'dgg 5/2/2011 15:36'!
testMultiAfterAdvice
	| mock asp pc |
	mock := PhMockClass new.
	asp := PhAspect new.
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'counter';
		context: #(receiver).  
	asp add: (PhAdvice new
		advice: [:context | context receiver minus];
		pointcut: pc;
		type: #after).
	asp add: (PhAdvice new
		advice: [:context | context receiver minus];
		pointcut: pc;
		type: #after).
	asp add: (PhAdvice new
		advice: [:context | context receiver minus];
		pointcut: pc;
		type: #after).
	
	[asp install.
	
	self assert: (mock counter = 0).
	self assert: (mock counter = -3)] ensure: [asp uninstall.]! !

!PhAspectTests methodsFor: 'as yet unclassified' stamp: 'dgg 2/20/2012 20:12'!
testMultiAroundAdvice
	| mock asp pc |
	mock := PhMockClass new.
	asp := PhAspect new.
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'plus';
		context: #(#receiver #proceed).
	asp add: (PhAdvice new
		advice: [:context | context receiver minus. context proceed];
		pointcut: pc;
		type: #around).
	asp add: (PhAdvice new
		advice: [:context | context receiver minus. context proceed];
		pointcut: pc;
		type: #around).
	asp add: (PhAdvice new
		advice: [:context | context receiver minus.];
		pointcut: pc;
		type: #around).
	
	[asp install.

	mock plus.
	self assert: (mock counter = -3)] ensure: [asp uninstall.]! !

!PhAspectTests methodsFor: 'as yet unclassified' stamp: 'dgg 5/2/2011 15:36'!
testMultiBeforeAdvice
	| mock asp pc |
	mock := PhMockClass new.
	asp := PhAspect new.
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'counter';
		context: #(#receiver).  
	asp add: (PhAdvice new
		advice: [:context | context receiver minus];
		pointcut: pc;
		type: #before).
	asp add: (PhAdvice new
		advice: [:context | context receiver minus];
		pointcut: pc;
		type: #before).
	asp add: (PhAdvice new
		advice: [:context | context receiver minus];
		pointcut: pc;
		type: #before).
	
	[asp install.
	
	self assert: (mock counter = -3)] ensure: [asp uninstall.]! !

!PhAspectTests methodsFor: 'pointcut-precedence' stamp: 'dgg 5/2/2011 15:37'!
testPointcutCombinePrecedenceAfter
	|pc aspect1 aspect2 aspect3 mock |
	mock := (PhMockClass new).
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'test';
		context: #(#receiver);
		precedence: #('PhMockAspect1' 'PhMockAspect2').  
	aspect1 := PhMockAspect1 new 
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | context receiver test: ('asp1', context receiver testString)];
			type: #after);
		precedence: #('PhMockAspect3' 'PhMockAspect2').
	aspect2 := PhMockAspect2 new add: ( PhAdvice new 
		pointcut: pc;
		advice: [:context | context receiver test: ('asp2', context receiver testString)];
		type: #after).
	aspect3 := PhMockAspect3 new add: (PhAdvice new  
		pointcut: pc;
		advice: [:context | context receiver test: ('asp3', context receiver testString)];
		type: #after).
	
	[aspect1 install.
	aspect2 install.
	aspect3 install.
	mock test.
	self assert: (mock test = 'asp3asp1asp2test')] ensure: [
		aspect1 uninstall.
		aspect2 uninstall.
		aspect3 uninstall].  ! !

!PhAspectTests methodsFor: 'pointcut-precedence' stamp: 'dgg 5/2/2011 15:38'!
testPointcutOverridePrecedenceAfter
	|pc aspect1 aspect2 aspect3 mock |
	mock := (PhMockClass new).
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'test';
		context: #(#receiver);
		overridePrecedence: #('PhMockAspect1' 'PhMockAspect2' 'PhMockAspect3').  
	aspect1 := PhMockAspect1 new 
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | context receiver test: ('asp1', context receiver testString)];
			type: #after);
		precedence: #('PhMockAspect3' 'PhMockAspect1' 'PhMockAspect2').
	aspect2 := PhMockAspect2 new add: ( PhAdvice new 
		pointcut: pc;
		advice: [:context | context receiver test: ('asp2', context receiver testString)];
		type: #after).
	aspect3 := PhMockAspect3 new add: (PhAdvice new  
		pointcut: pc;
		advice: [:context | context receiver test: ('asp3', context receiver testString)];
		type: #after).
	
	[
	aspect1 install.
	aspect2 install.
	aspect3 install.
	mock test.
	self assert: (mock test = 'asp1asp2asp3test')] ensure: [
		aspect1 uninstall.
		aspect2 uninstall.
		aspect3 uninstall].  ! !

!PhAspectTests methodsFor: 'pointcut-precedence' stamp: 'dgg 5/2/2011 15:39'!
testPointcutPrecedenceAfter
	|pc aspect1 aspect2 aspect3 mock |
	mock := (PhMockClass new).
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'test';
		context: #(#receiver);
		precedence: #('PhMockAspect1' 'PhMockAspect2').  
	aspect1 := PhMockAspect1 new 
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | context receiver test: ('asp1', context receiver testString)];
			type: #after);
		precedence: #('PhMockAspect3' 'PhMockAspect2' 'PhMockAspect1').
	aspect2 := PhMockAspect2 new add: ( PhAdvice new 
		pointcut: pc;
		advice: [:context | context receiver test: ('asp2', context receiver testString)];
		type: #after).
	aspect3 := PhMockAspect3 new add: (PhAdvice new  
		pointcut: pc;
		advice: [:context | context receiver test: ('asp3', context receiver testString)];
		type: #after).
	
	[
	aspect1 install.
	aspect2 install.
	aspect3 install.
	mock test.
	self assert: (mock test = 'asp3asp1asp2test')] ensure: [
	aspect1 uninstall.
	aspect2 uninstall.
	aspect3 uninstall].  ! !

!PhAspectTests methodsFor: 'precedence' stamp: 'dgg 5/2/2011 15:39'!
testPrecedenceAfter
	|pc aspect1 aspect2 aspect3 mock |
	mock := (PhMockClass new).
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'test';
		context: #(#receiver).  
	aspect1 := PhMockAspect1 new 
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | context receiver test: ('asp1', context receiver testString)];
			type: #after);
		precedence: #('PhMockAspect3' 'PhMockAspect1' 'PhMockAspect2').
	aspect2 := PhMockAspect2 new add: ( PhAdvice new 
		pointcut: pc;
		advice: [:context | context receiver test: ('asp2', context receiver testString)];
		type: #after).
	aspect3 := PhMockAspect3 new add: (PhAdvice new  
		pointcut: pc;
		advice: [:context | context receiver test: ('asp3', context receiver testString)];
		type: #after).
	
	[
	aspect1 install.
	aspect2 install.
	aspect3 install.
	mock test.
	self assert: (mock test = 'asp3asp1asp2test')] ensure: [
		aspect1 uninstall.
		aspect2 uninstall.
		aspect3 uninstall].  ! !

!PhAspectTests methodsFor: 'precedence' stamp: 'dgg 2/20/2012 20:10'!
testPrecedenceAround
	|pc aspect1 aspect2 aspect3 |
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'test';
		context: #(proceed).
	aspect1 := PhMockAspect1 new 
		add: (PhAdvice new  
			pointcut: pc;
			advice: [:ctx | 'asp1',ctx proceed.];
			type: #around);
		precedence: #('PhMockAspect3' 'PhMockAspect1' 'PhMockAspect2').
	aspect2 := PhMockAspect2 new add: (PhAdvice new  
		pointcut: pc;
		advice: [:ctx | 'asp2'];
		type: #around).
	aspect3 := PhMockAspect3 new add: (PhAdvice new
		pointcut: pc;
		advice: [:ctx | 'asp3',ctx proceed];
		type: #around).
		
	
	[
	aspect1 install.
	aspect2 install.
	aspect3 install.
	self assert: ((PhMockClass new) test = 'asp3asp1asp2')] ensure: [
		aspect1 uninstall.
		aspect2 uninstall.
		aspect3 uninstall].  ! !

!PhAspectTests methodsFor: 'precedence' stamp: 'dgg 5/2/2011 15:41'!
testPrecedenceBefore
	|pc aspect1 aspect2 aspect3 |
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'test';
		context: #(#receiver).
	aspect1 := PhMockAspect1 new 
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | context receiver test: ('asp1', context receiver testString)];
			type: #before);
		precedence: #('PhMockAspect3' 'PhMockAspect1' 'PhMockAspect2').
	aspect2 := PhMockAspect2 new add: (PhAdvice new
		pointcut: pc;
		advice: [:context | context receiver test: ('asp2', context receiver testString)];
		type: #before).
	aspect3 := PhMockAspect3 new add: (PhAdvice new
		pointcut: pc;
		advice: [:context | context receiver test: ('asp3', context receiver testString)];
		type: #before).
	
	  
	[
	aspect1 install.
	aspect2 install.
	aspect3 install.
	self assert: ((PhMockClass new) test = 'asp2asp1asp3test')] ensure: [
		aspect1 uninstall.
		aspect2 uninstall.
		aspect3 uninstall].  ! !

!PhAspectTests methodsFor: 'precedence' stamp: 'dgg 2/20/2012 20:15'!
testPrecedenceMultiRule
	|pc aspect1 aspect2 aspect3 |
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'test';
		context: #(#proceed).  
	aspect1 := PhMockAspect1 new 
		add: (PhAdvice new
			pointcut: pc;
			advice: [:ctx | 'asp1', ctx proceed];
			type: #around);
		precedence: #('PhMockAspect3' 'PhMockAspect1').
	aspect2 := PhMockAspect2 new 
		add: (PhAdvice new
			pointcut: pc;
			advice: [:ctx | 'asp2'];
			type: #around);
		precedence: #('PhMockAspect1' 'PhMockAspect2').
	aspect3 := PhMockAspect3 new 
		add: (PhAdvice new
			pointcut: pc;
			advice: [:ctx | 'asp3',ctx proceed];
			type: #around);
		precedence: #('PhMockAspect3' 'PhMockAspect2').
	
	[
	aspect1 install.
	aspect2 install.
	aspect3 install.
	self assert: ((PhMockClass new) test = 'asp3asp1asp2')] ensure: [
		aspect1 uninstall.
		aspect2 uninstall.
		aspect3 uninstall].  ! !

TestCase subclass: #PhAspectWeaverTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

!PhAspectWeaverTests methodsFor: 'as yet unclassified' stamp: 'dgg 10/7/2011 17:19'!
testInstalledAspects
	| asp |
	asp := PhAspect new
		add: (PhAdvice new
			pointcut: (PhPointcut new
				receivers: 'PhMockClass';
				selectors: 'foo');
			advice: [:ctx | 'foo'];
			type: #around).
	[asp install.
	self assert: (PhAspectWeaver installedAspects includes: asp)] ensure: [
	asp uninstall.
	self deny: (PhAspectWeaver installedAspects includes: asp)]! !

!PhAspectWeaverTests methodsFor: 'as yet unclassified' stamp: 'dgg 6/23/2011 16:30'!
testInstalledMembranes
	| membrane |
	membrane := PhMembrane new
		pointcut: (PhPointcut new
			receivers: 'PhMockClass';
			selectors: 'foo').
	[membrane install.
	self assert: (PhAspectWeaver installedMembranes includes: membrane)] ensure: [
	membrane uninstall.
	self deny: (PhAspectWeaver installedMembranes includes: membrane)]! !

!PhAspectWeaverTests methodsFor: 'as yet unclassified' stamp: 'dgg 5/1/2011 20:06'!
testMembraneMethodWrapperDictionary
	| membrane wrappers targetClass targetSelectors |
	membrane := PhMembrane new
		pointcut:
			(PhPointcut new
				receivers: 'PhMockClass';
				selectors: #('foo' 'test' 'bar')).
	[membrane install.
	wrappers := PhAspectWeaver classVarNamed: 'WrapperList'.
	self assert: wrappers size = 3.
	self assert: (wrappers includesKey: (Array with: PhMockClass with: #foo)).
	self assert: (wrappers includesKey: (Array with: PhMockClass with: #test)).
	self assert: (wrappers includesKey: (Array with: PhMockClass with: #bar)) ] ensure: [ membrane uninstall ]! !

!PhAspectWeaverTests methodsFor: 'as yet unclassified' stamp: 'dgg 5/1/2011 16:30'!
testMethodWrapperDictionary
	| asp wrappers targetClass targetSelectors |
	
	"asp := PhAspect new.
	asp add: (PhAdvice new
				advice: [:context | 'foo'];
				pointcut: (PhPointcut new
					receivers: 'PhMockClass';
					selectors: 'foo');
				type: #before).
				
	 asp add: (PhAdvice new
				advice: [:context | 'foo'];
				pointcut: (PhPointcut new
					receivers: 'PhMockClass';
					selectors: 'test');
				type: #before).
	
	 asp add: (PhAdvice new
				advice: [:context | 'foo'];
				pointcut: (PhPointcut new
					receivers: 'PhMockClass';
					selectors: 'bar');
				type: #before).
				
	asp install.
	[wrappers := PhAspectWeaver classVarNamed: 'WrapperList'.
	self assert: (wrappers size = 3).
	self assert: (wrappers includesKey: (Array with: PhMockClass with: #foo)).
	self assert: (wrappers includesKey: (Array with: PhMockClass with: #test)).
	self assert: (wrappers includesKey: (Array with: PhMockClass with: #bar)).
	] ensure: [asp uninstall]"! !

!PhAspectWeaverTests methodsFor: 'as yet unclassified' stamp: 'dgg 5/1/2011 16:30'!
testMethodWrapperGeneration
"	| asp wrappers targetClass targetSelectors |
	
	asp := PhAspect new.
	asp add: (PhAdvice new
				advice: [:context | 'foo'];
				pointcut: (PhPointcut new
					receivers: 'PhMockClass';
					selectors: 'foo');
				type: #before).
				
	 asp add: (PhAdvice new
				advice: [:context | 'foo'];
				pointcut: (PhPointcut new
					receivers: 'PhMockClass';
					selectors: 'test');
				type: #before).
	
	 asp add: (PhAdvice new
				advice: [:context | 'foo'];
				pointcut: (PhPointcut new
					receivers: 'PhMockClass';
					selectors: 'bar');
				type: #before).
				
	asp install.
	[wrappers := PhAspectWeaver classVarNamed: 'WrapperList'.
	self assert: ((wrappers at: (Array with: PhMockClass with: #foo)) reference classSymbol = #PhMockClass).
	self assert: ((wrappers at: (Array with: PhMockClass with: #test)) reference classSymbol = #PhMockClass).
	self assert: ((wrappers at: (Array with: PhMockClass with: #bar)) reference classSymbol = #PhMockClass).
	self assert: ((wrappers at: (Array with: PhMockClass with: #foo)) reference methodSymbol = #foo).
	self assert: ((wrappers at: (Array with: PhMockClass with: #test)) reference methodSymbol = #test).
	self assert: ((wrappers at: (Array with: PhMockClass with: #bar)) reference methodSymbol = #bar).
	] ensure: [asp uninstall]"! !

TestCase subclass: #PhCflowPointcutTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

!PhCflowPointcutTests methodsFor: 'as yet unclassified' stamp: 'dgg 10/15/2011 23:44'!
testNestedCFlow
       | tocalltestcall nestedtestcall testcall asp |
       tocalltestcall := PhPointcut new
               receivers: 'PhMockClass';
               selectors: 'toCallTest'.
       nestedtestcall := PhPointcut new
               receivers: 'PhMockClass';
               selectors: 'test';
               inCflowOf: tocalltestcall.
       testcall := PhPointcut new
               receivers: 'PhMockClass';
               selectors: 'test'.
       asp := PhAspect new add:
               (PhAdvice new
                       pointcut: (testcall & (nestedtestcall not));
                       advice: [:ctx | 'direct'];
                       type: #around).
       [asp install.
       self assert: (PhMockClass new test = 'direct').
       self assert: (PhMockClass new toCallTest = 'test')] ensure: [asp uninstall]! !

!PhCflowPointcutTests methodsFor: 'as yet unclassified' stamp: 'dgg 1/25/2012 23:30'!
testNestedCFlow2
       | tocalltestcall directCall asp |
       tocalltestcall := PhPointcut new
               receivers: 'PhMockClass';
               selectors: 'toCallTest'.
       directCall := PhPointcut new
               receivers: 'PhMockClass';
               selectors: 'test';
               notInCflowOf: tocalltestcall.

       asp := PhAspect new add:
               (PhAdvice new
                       pointcut: directCall;
                       advice: [:ctx | 'direct'];
                       type: #around).
       [asp install.
       self assert: (PhMockClass new test = 'direct').
       self assert: (PhMockClass new toCallTest = 'test')] ensure: [asp uninstall]! !

!PhCflowPointcutTests methodsFor: 'as yet unclassified' stamp: 'dgg 10/10/2011 14:32'!
testSimpleCFlow
	| pc asp |
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'test';
		inCflowOf: (PhPointcut receivers: 'PhMockClass' selectors: 'toCallTest').
	asp := PhAspect new add: 
		(PhAdvice new 
			pointcut: pc;
			advice: [:ctx | 'inCflowOf'];
			type: #around).
	[asp install.
	self assert: (PhMockClass new test = 'test').
	self assert: (PhMockClass new toCallTest = 'inCflowOf')] ensure: [asp uninstall]! !

TestCase subclass: #PhClassModifierTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

!PhClassModifierTests methodsFor: 'as yet unclassified' stamp: 'dgg 9/29/2011 14:26'!
testAddNewClassMethod
	| aClassModifier |
	[aClassModifier := PhClassModifier new on: (PhPointcut receivers: 'PhMockClass' selectors: #any asParser).
	self deny: (PhMockClass theMetaClass localSelectors includes: #theTestMethod).  
	aClassModifier addNewClassMethod: 'theTestMethod
		^''success'' '.  
	aClassModifier install.
	self assert: (PhMockClass theTestMethod = 'success').] ensure: [
	aClassModifier uninstall.
	self deny: (PhMockClass theMetaClass localSelectors includes: #theTestMethod).]
	    ! !

!PhClassModifierTests methodsFor: 'as yet unclassified' stamp: 'dgg 9/29/2011 14:26'!
testAddNewClassVar
	| aClassModifier |
	[aClassModifier := PhClassModifier new on: (PhPointcut receivers: 'PhMockClass' selectors: #any asParser).
	self deny: (PhMockClass hasClassVarNamed: 'Asdf').
	aClassModifier addNewClassVar: 'Asdf'.
	aClassModifier install.
	self assert: (PhMockClass hasClassVarNamed: 'Asdf').] ensure: [
	aClassModifier uninstall.
	self deny: (PhMockClass hasClassVarNamed: 'Asdf')]
	    ! !

!PhClassModifierTests methodsFor: 'as yet unclassified' stamp: 'dgg 9/29/2011 14:27'!
testAddNewInstanceMethod
	| aClassModifier |
	[aClassModifier := PhClassModifier new on: (PhPointcut receivers: 'PhMockClass' selectors: #any asParser ).
	self deny: (PhMockClass theNonMetaClass localSelectors includes: #theTestMethod).  
	aClassModifier addNewInstanceMethod: 'theTestMethod
		^''success'' '.  
	aClassModifier install.
	self assert: (PhMockClass new theTestMethod = 'success').] ensure: [
	aClassModifier uninstall.
	self deny: (PhMockClass theNonMetaClass localSelectors includes: #theTestMethod).]
	    ! !

!PhClassModifierTests methodsFor: 'as yet unclassified' stamp: 'dgg 9/29/2011 14:27'!
testAddNewInstanceVar
	| aClassModifier |
	[aClassModifier := PhClassModifier new on: (PhPointcut receivers: 'PhMockClass' selectors: #any asParser).
	self deny: (PhMockClass hasInstVarNamed: 'asdf').
	aClassModifier addNewInstanceVar: 'asdf'.
	aClassModifier install.
	self assert: (PhMockClass hasInstVarNamed: 'asdf').] ensure: [
	aClassModifier uninstall.
	self deny: (PhMockClass hasInstVarNamed: 'asdf')]
	    ! !

!PhClassModifierTests methodsFor: 'as yet unclassified' stamp: 'dgg 9/29/2011 14:27'!
testCompleModification
	| aClassModifier mock |
	[aClassModifier := PhClassModifier new on: (PhPointcut receivers: 'PhMockClass' selectors: #any asParser).
	aClassModifier
		addNewInstanceVar: 'xxxVar';
		addNewInstanceMethod: 'plusXXX
			xxxVar := self xxxVar +1';
		addNewInstanceMethod: 'xxxVar
			^ xxxVar ifNil: [xxxVar := 0]'.
	aClassModifier install.
	mock := PhMockClass new.
	self assert: (mock xxxVar = 0).
	mock plusXXX.
	self assert: (mock xxxVar = 1).] ensure: [
	aClassModifier uninstall.] 	
	! !

!PhClassModifierTests methodsFor: 'accessing' stamp: 'dgg 10/7/2011 18:37'!
testMultipointcut
	| pc cm |
	pc := PhPointcut new
		receivers: #('PhMockClass' 'PhMockAspectFoo');
		selectors: #any asParser.
	
	cm := PhClassModifier new on: pc.
	cm 
		addNewInstanceVar: 'foobarVar';
		addNewInstanceMethod: 'foobarMethod ^true'.
	[cm install.
	self assert: (PhMockClass hasInstVarNamed: 'foobarVar').
	self assert: (PhMockClass methodDict includesKey: #foobarMethod).
	self assert: (PhMockClass new perform: #foobarMethod).
	self assert: (PhMockAspectFoo hasInstVarNamed: 'foobarVar').
	self assert: (PhMockAspectFoo methodDict includesKey: #foobarMethod).
	self assert: (PhMockAspectFoo new perform: #foobarMethod).] ensure: [cm uninstall]! !

!PhClassModifierTests methodsFor: 'accessing' stamp: 'dgg 10/7/2011 18:36'!
testPointcutWithSubclasses
	| pc cm |
	pc := PhPointcut new
		receivers: #('PhMockClass+' 'PhMockAspectFoo+');
		selectors: #any asParser.
	
	cm := PhClassModifier new on: pc.
	cm 
		addNewInstanceVar: 'foobarVar';
		addNewInstanceMethod: 'foobarMethod ^true'.
	[cm install.
	self assert: (PhMockClass hasInstVarNamed: 'foobarVar').
	self assert: (PhMockClass methodDict includesKey: #foobarMethod).
	self assert: (PhMockClass new perform: #foobarMethod).
	self assert: (PhMockSubclass allSelectors includes: #foobarMethod).
	self assert: (PhMockSubclass new perform: #foobarMethod).
	self assert: (PhMockAspectFoo hasInstVarNamed: 'foobarVar').
	self assert: (PhMockAspectFoo methodDict includesKey: #foobarMethod).
	self assert: (PhMockAspectFoo new perform: #foobarMethod).
	self assert: (PhMockFooAspect5 allSelectors includes: #foobarMethod).
	self assert: (PhMockFooAspect5 new perform: #foobarMethod).] ensure: [cm uninstall]
	
	
	! !

!PhClassModifierTests methodsFor: 'as yet unclassified' stamp: 'dgg 9/29/2011 14:17'!
testSimplePointcutAsArgument
	| pc cm |
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: #any asParser.
	
	cm := PhClassModifier new on: pc.
	cm 
		addNewInstanceVar: 'foobarVar';
		addNewInstanceMethod: 'foobarMethod ^true'.
	[cm install.
	self assert: (PhMockClass hasInstVarNamed: 'foobarVar').
	self assert: (PhMockClass methodDict includesKey: #foobarMethod).
	self assert: (PhMockClass new perform: #foobarMethod)] ensure: [cm uninstall]
	
	
	! !

TestCase subclass: #PhDynamicTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

!PhDynamicTests methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 4/14/2011 13:42'!
aPhMockClassParserWithPPPInstanceVar
	^ 'Object subclass: #PhMockClass
	instanceVariableNames: ''counter testString ppp''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''Phantom-Tests''' asParser.! !

!PhDynamicTests methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 4/12/2011 21:47'!
addFooOnPhNoOneMock
	(SmalltalkImage current at: #PhNoOneMock) compile: 'foo ^''bar'' '.! !

!PhDynamicTests methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 4/12/2011 21:47'!
checkNoOneFoo
	self assert: ((SmalltalkImage current at: #PhNoOneMock) new foo = 'foo')! !

!PhDynamicTests methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 4/12/2011 21:47'!
checkNoOneHash
	self assert: ((SmalltalkImage current at: #PhNoOneMock) new hash = 'foo')! !

!PhDynamicTests methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 4/12/2011 22:01'!
checkPhMockClassFoo
	self assert: ((SmalltalkImage current at: #PhMockClass) new foo = 'foo')! !

!PhDynamicTests methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 4/14/2011 14:11'!
checkPhMockSubclassFoo
	self assert: ((SmalltalkImage current at: #PhMockSubclass) new foo = 'foo')! !

!PhDynamicTests methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 4/14/2011 13:46'!
createMockClassSubclass
	PhMockClass subclass: #PhMockSubclass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'! !

!PhDynamicTests methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 4/14/2011 13:46'!
createNoOneClass
	Object subclass: #PhNoOneMock
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'Phantom-Tests'.! !

!PhDynamicTests methodsFor: 'as yet unclassified' stamp: 'dgg 5/1/2011 19:47'!
removeMockSubclass
	(SmalltalkImage current at: #PhMockSubclass) removeFromSystem.! !

!PhDynamicTests methodsFor: 'as yet unclassified' stamp: 'dgg 5/1/2011 19:47'!
removeNoOneClass
	(SmalltalkImage current at: #PhNoOneMock) removeFromSystem.! !

!PhDynamicTests methodsFor: 'as yet unclassified' stamp: 'dgg 10/7/2011 17:20'!
testClassModification
	| asp |
	[
	asp := PhAspect new add: (PhAdvice new
		pointcut: (PhPointcut new
			receivers: self aPhMockClassParserWithPPPInstanceVar;
			selectors: 'foo');
		advice: [:ctx | 'foo'];
		type: #around).
	
	asp install.
	self deny: (PhMockClass new foo = 'foo').  
	PhMockClass addInstVarNamed: 'ppp'.
	self checkPhMockClassFoo.
	] ensure: [
	asp uninstall.
	PhMockClass removeInstVarNamed: 'ppp'.]! !

!PhDynamicTests methodsFor: 'as yet unclassified' stamp: 'dgg 10/7/2011 17:20'!
testCreateMethod
	| asp |
	[
	asp := PhAspect new add: (PhAdvice new
		pointcut: (PhPointcut new
			receivers: 'PhNoOneMock';
			selectors: 'foo');
		advice: [:ctx | 'foo'];
		type: #around).
	
	asp install.  
	self addFooOnPhNoOneMock.
	self checkNoOneFoo.
	] ensure: [
	asp uninstall.
	(SmalltalkImage current at: #PhNoOneMock) removeSelector: #foo]! !

!PhDynamicTests methodsFor: 'as yet unclassified' stamp: 'dgg 10/7/2011 17:20'!
testMethodModification
	| asp |
	[
	asp := PhAspect new add: (PhAdvice new
		pointcut: (PhPointcut new
			receivers: 'PhMockClass';
			selectors: 'foo');
		advice: [:ctx | 'foo'];
		type: #around).
	
	asp install.
	self checkPhMockClassFoo.
	PhMockClass compile: 'foo ^''blah'' '.
	self checkPhMockClassFoo.
	] ensure: [
	asp uninstall.
	PhMockClass compile: 'foo'.]! !

!PhDynamicTests methodsFor: 'as yet unclassified' stamp: 'dgg 10/7/2011 17:20'!
testNewClassOnSystem
	| asp |
	[self removeNoOneClass.
	
	asp := PhAspect new add: (PhAdvice new
		pointcut: (PhPointcut new
			receivers: 'PhNoOneMock';
			selectors: 'hash');
		advice: [:ctx | 'foo'];
		type: #around).
	
	asp install.
	self createNoOneClass.
	self checkNoOneHash.
	] ensure: [asp uninstall]
	! !

!PhDynamicTests methodsFor: 'as yet unclassified' stamp: 'dgg 10/7/2011 17:20'!
testNewSubClassOnSystem
	| asp |
	[
	self removeMockSubclass.
	asp := PhAspect new add: (PhAdvice new
		pointcut: (PhPointcut new
			receivers: 'PhMockClass+';
			selectors: 'foo');
		advice: [:ctx | 'foo'];
		type: #around).
	
	asp install.
	self createMockClassSubclass.
	self checkPhMockSubclassFoo.
	] ensure: [asp uninstall]
	! !

!PhDynamicTests methodsFor: 'as yet unclassified' stamp: 'dgg 10/7/2011 17:21'!
testRenameClass
	| asp |
	[SmalltalkImage current renameClassNamed: 'PhNoOneMock' as: 'PhNoOneOneMock'.
	
	asp := PhAspect new add: (PhAdvice new
		pointcut: (PhPointcut new
			receivers: 'PhNoOneMock';
			selectors: 'hash');
		advice: [:ctx | 'foo'];
		type: #around).
	asp install.
	SmalltalkImage current renameClassNamed: 'PhNoOneOneMock' as: 'PhNoOneMock'.
	self checkNoOneHash] ensure: [asp uninstall]! !

!PhDynamicTests methodsFor: 'as yet unclassified' stamp: 'dgg 10/7/2011 17:21'!
testUninstallAfterClassRemove
	| asp |
	[
	asp := PhAspect new add: (PhAdvice new
		pointcut: (PhPointcut new
			receivers: 'PhNoOneMock';
			selectors: 'hash');
		advice: [:ctx | 'foo'];
		type: #around).
	
	asp install.
	self checkNoOneHash.
	self removeNoOneClass.
	self uninstall: asp.  
	self createNoOneClass.
	] ensure: [
	asp uninstall.
	self createNoOneClass.]
	! !

!PhDynamicTests methodsFor: 'as yet unclassified' stamp: 'dgg 10/7/2011 17:21'!
testUninstallAfterMethodRemove
	| asp |
	[
	asp := PhAspect new add: (PhAdvice new
		pointcut: (PhPointcut new
			receivers: 'PhNoOneMock';
			selectors: 'foo');
		advice: [:ctx | 'foo'];
		type: #around).
	
	asp install.
	self addFooOnPhNoOneMock.
	self checkNoOneFoo.
	(SmalltalkImage current at: #PhNoOneMock) removeSelector: #foo.
	self uninstall: asp.
	] ensure: [
	asp uninstall.
	(SmalltalkImage current at: #PhNoOneMock) removeSelector: #foo]! !

!PhDynamicTests methodsFor: 'as yet unclassified' stamp: 'dgg 5/2/2011 15:46'!
uninstall: asp
	asp uninstall.! !

TestCase subclass: #PhIfPointcutTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

!PhIfPointcutTests methodsFor: 'as yet unclassified' stamp: 'dgg 11/12/2011 20:13'!
testCombinedIfPointcut
	| alwaysFalsePc alwaysTruePc orPc andPc orAsp andAsp |
	alwaysFalsePc := PhPointcut
		receivers: 'PhMockClass'
		selectors: 'test'
		if: [:ctx | false].
	alwaysTruePc := PhPointcut
		receivers: 'PhMockClass'
		selectors: 'test'
		if: [:ctx | true].
	orPc := alwaysFalsePc | alwaysTruePc.
	andPc := alwaysTruePc & alwaysFalsePc.
	
	orAsp := PhAspect new
		add: (PhAdvice
			around: orPc
			advice: [:ctx | 'orOk']).
	andAsp := PhAspect new
		add: (PhAdvice
			around: andPc
			advice: [:ctx | 'orOk']).
	[orAsp install.
	self assert: (PhMockClass new test = 'orOk')] ensure: [orAsp uninstall].
	[andAsp install.
	self assert: (PhMockClass new test = 'test')] ensure: [andAsp uninstall].! !

!PhIfPointcutTests methodsFor: 'as yet unclassified' stamp: 'dgg 1/25/2012 17:06'!
testCombinedIfPointcut2
	| pc ifpc adv asp |
	
	pc := PhPointcut receivers: 'PhMockClass' selectors: 'test'.
	ifpc:= PhPointcut receivers: 'PhMockClass' selectors: 'test:' context: #(arguments) if: [:ctx | (ctx arguments at: 1) = 'true'].
	adv := PhAdvice
		around: (ifpc | pc)
		advice: [:ctx |
			'ok'].
	asp := PhAspect new add: adv.
	[asp install.
	self assert: (PhMockClass new test = 'ok').
	self deny: ((PhMockClass new test: 'false') = 'ok').
	self assert: ((PhMockClass new test: 'true') = 'ok')] ensure: [asp uninstall]! !

!PhIfPointcutTests methodsFor: 'as yet unclassified' stamp: 'dgg 11/12/2011 16:59'!
testContextIfPointcut
	| asp |
	asp := PhAspect new
		add: (PhAdvice
			around: (PhPointcut
				receivers: 'PhMockClass'
				selectors: 'echo:'
				context: #(arguments)
				if: [:ctx |
					(ctx arguments at: 1) = 'active'])
			advice: [:ctx | 'ok']).
	[asp install.
	self assert: ((PhMockClass new echo: 'active') = 'ok').
	self assert: ((PhMockClass new echo: 'test') = 'test')] ensure: [asp uninstall]! !

!PhIfPointcutTests methodsFor: 'as yet unclassified' stamp: 'dgg 11/12/2011 16:55'!
testEmptyContextFalseIfPointcut
	| asp |
	asp := PhAspect new
		add: (PhAdvice
			around: (PhPointcut
				receivers: 'PhMockClass'
				selectors: 'test'
				if: [:ctx | false])
			advice: [:ctx | 'ok']).
	[asp install.
	self assert: (PhMockClass new test = 'test').
	self deny: (PhMockClass new test = 'ok')] ensure: [asp uninstall]! !

!PhIfPointcutTests methodsFor: 'as yet unclassified' stamp: 'dgg 11/12/2011 16:54'!
testEmptyContextTrueIfPointcut
	| asp |
	asp := PhAspect new
		add: (PhAdvice
			around: (PhPointcut
				receivers: 'PhMockClass'
				selectors: 'test'
				if: [:ctx | true])
			advice: [:ctx | 'ok']).
	[asp install.
	self assert: (PhMockClass new test = 'ok').
	self deny: (PhMockClass new test = 'test')] ensure: [asp uninstall]! !

TestCase subclass: #PhMembraneTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

!PhMembraneTests methodsFor: 'as yet unclassified' stamp: 'dgg 10/7/2011 17:21'!
testAdvisedMembrane
	| membrane1 membrane2  asp1 asp2 pc |
	[pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'test'.
	membrane1 := PhMembrane new.
	asp1 := PhAspect new
		add: (PhAdvice new
			advice: [:ctx | Transcript show: (PhMockClass new test); cr];
			pointcut: pc;
			type: #after).
	asp1 registerOn: membrane1.
	membrane2 := PhMembrane new advise: membrane1.
	asp2 := PhAspect new
		add: (PhAdvice new
			advice: [:ctx | Transcript show: 'Advised'; cr];
			pointcut: pc;
			type: #after).
	asp2 registerOn: membrane2.
	membrane1 install. 
	self assert: (PhMockClass new test = 'test').
	] ensure: [
	membrane1 uninstall]! !

!PhMembraneTests methodsFor: 'as yet unclassified' stamp: 'dgg 10/7/2011 17:21'!
testMembraneVisibility
	| asp membrane aspectMembrane pc1 pc2 |
	pc1 := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'test'.
	pc2 := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'testString'.
	
	membrane := PhMembrane new pointcut: pc1.
	aspectMembrane := PhMembrane new advise: membrane.
	asp := PhAspect new
		add: (PhAdvice new
			pointcut: (pc1 | pc2);
			advice: [:ctx | 'foo'];
			type: #around).
	[asp registerOn: aspectMembrane.
	membrane install.
	self assert: (PhMockClass new test = 'foo').
	self deny: (PhMockClass new testString = 'foo').
	self assert: (PhMockClass new testString = 'test')] ensure: [membrane uninstall]    ! !

!PhMembraneTests methodsFor: 'as yet unclassified' stamp: 'dgg 10/7/2011 17:22'!
testReentrancy
	| membrane asp pc |
	[pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'test'.
	membrane := PhMembrane new.
	asp := PhAspect new
		add: (PhAdvice new
			advice: [:ctx | Transcript show: (PhMockClass new test); cr];
			pointcut: pc;
			type: #after).
	asp registerOn: membrane.
	membrane install.
	self assert: (PhMockClass new test = 'test').
	] ensure: [membrane uninstall.]! !

TestCase subclass: #PhObjectDeployedMembraneTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

!PhObjectDeployedMembraneTests methodsFor: 'as yet unclassified' stamp: 'dgg 9/18/2011 01:28'!
tearDown
	super tearDown.
	SmalltalkImage current garbageCollect.! !

!PhObjectDeployedMembraneTests methodsFor: 'as yet unclassified' stamp: 'dgg 5/30/2011 01:46'!
testBlockObjectDeployedMembrane
"	| mock block membrane membrane2 asp |
	asp := PhAspect new
		add: (PhAdvice new
			pointcut: (PhPointcut new
				receivers: 'PhMockClass';
				selectors: 'test');
			advice: ['foo'];
			type: #around).
	membrane := PhMembrane new.
	membrane2 := PhMembrane new advise: membrane.
	asp registerOn:  membrane2.
	mock := PhMockClass new.
	block := [mock test].
	membrane deployOn:  block.
	self assert: (mock test = 'test').
	self assert: (block value = 'foo')."! !

!PhObjectDeployedMembraneTests methodsFor: 'as yet unclassified' stamp: 'dgg 10/7/2011 17:22'!
testObjectDeployedMembrane
	| mock1 mock2 membrane asp |
	asp := PhAspect new
		add: (PhAdvice new
			pointcut: (PhPointcut new
				receivers: 'PhMockClass';
				selectors: 'test');
			advice: [:ctx | 'foo'];
			type: #around).
	membrane := PhMembrane new.
	asp registerOn:  membrane.
	mock1 := PhMockClass new.
	mock2 := PhMockClass new.
	membrane deployOn: mock1.
	self deny: (mock1 test = 'foo').
	self assert: (mock2 test = 'test').! !

!PhObjectDeployedMembraneTests methodsFor: 'as yet unclassified' stamp: 'dgg 10/7/2011 17:22'!
testObjectDeployedMembrane2
	| mock1 mock2 membrane membrane2 asp |
	asp := PhAspect new
		add: (PhAdvice new
			pointcut: (PhPointcut new
				receivers: 'PhMockClass';
				selectors: 'test');
			advice: [:ctx | 'foo'];
			type: #around).
	membrane := PhMembrane new.
	membrane2 := PhMembrane new advise: membrane.
	asp registerOn:  membrane2.
	mock1 := PhMockMockClass new.
	mock2 := PhMockClass new.
	membrane deployOn: mock1.
	self assert: (mock1 callTestMock = 'foo').
	self assert: (mock2 test = 'test').
	self assert: (PhMockMockClass new callTestMock = 'test')! !

!PhObjectDeployedMembraneTests methodsFor: 'as yet unclassified' stamp: 'dgg 10/7/2011 17:22'!
testObjectDeployedMembrane3
	| mock1 mock2 membrane asp aspMembrane |
	asp := PhAspect new
		add: (PhAdvice new
			pointcut: (PhPointcut new
				receivers: 'PhMockClass';
				selectors: 'test');
			advice: [:ctx | 'foo'];
			type: #around).
	membrane := PhMembrane new.
	aspMembrane := PhMembrane new advise: membrane.
	asp registerOn:  aspMembrane.
	mock1 := PhMockClass new.
	mock2 := PhMockClass new.
	membrane deployOn: mock1.
	self assert: (mock1 test = 'foo').
	self assert: (mock2 test = 'test').! !

TestCase subclass: #PhPointcutTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

!PhPointcutTests methodsFor: 'context exposure' stamp: 'dgg 5/2/2011 15:47'!
testAfterAdvicesExposure
	| asp adv1 adv2 adv3 pc |
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'foo';
		context: #(#advice).
	
	
	
	adv1 := PhAdvice new
		pointcut: pc;
		advice: [:context | 'foo'];
		type: #after.
	adv2 := PhAdvice new
		pointcut: pc;
		advice: [:context | 'bar'];
		type: #after.
	adv3 := PhAdvice new
		pointcut: pc;
		advice: [:context | 'foobar'];
		type: #after.    
	
	asp := PhAspect new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | context afterAdvice];
			type: #around);
		add: adv1;
		add: adv2;
		add: adv3.
	[asp install.
	
	self assert: (PhMockClass new foo includes: adv1).
	self assert: (PhMockClass new foo includes: adv2).
	self assert: (PhMockClass new foo includes: adv3).
	self assert: (PhMockClass new foo size = 3)] ensure: [asp uninstall.]! !

!PhPointcutTests methodsFor: 'composing' stamp: 'dgg 9/18/2011 17:37'!
testAnd
	| pc1 pc2 targetSet |
	pc1 := PhPointcut new
		receivers: 'Morph';
		selectors: 'mouseEnter:'.
		
	pc2 := PhPointcut new
		receivers: 'Morph+';
		selectors: 'mouseEnter:'.
	targetSet := Set new.
	targetSet add: (Array with: Morph with: #mouseEnter:).
	self assert: ( (pc1 & pc2) getPointcuts = targetSet )          ! !

!PhPointcutTests methodsFor: 'matching' stamp: 'dgg 6/9/2011 14:12'!
testArgumentKeywordSelectorWithMiddleStringMatch
	| set pc |
	pc := (PhPointcut new)
		receivers: 'SequenceableCollection';
		localSelectors: '_:to:_:'.
	set := Set new.
	SequenceableCollection localSelectors
		do: [ :sel | 
			((sel isKeyword) and: [((sel findBetweenSubStrs: ':') size = 3) and: [(((sel findBetweenSubStrs: ':') at: 2) = 'to')]])
				ifTrue: [ set add: (Array with: SequenceableCollection with: sel) ] ].
	self deny: (set isEmpty).
	self assert: pc getPointcuts = set! !

!PhPointcutTests methodsFor: 'context exposure' stamp: 'dgg 5/2/2011 15:48'!
testArgumentsExposure
	| pc asp |
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'test:';
		context: #(#arguments).
	
	asp := PhAspect new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | context arguments at: 1];
			type: #around).
	
	[
	asp install.
	
	self assert: ((PhMockClass new test: 'argstests') = 'argstests')] ensure: [asp uninstall].    
		! !

!PhPointcutTests methodsFor: 'context exposure' stamp: 'dgg 5/2/2011 15:48'!
testAroundAdvicesExposure
	| asp adv1 adv2 adv3 pc |
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'foo';
		context: #(#advice).
	
	adv1 := PhAdvice new
			pointcut: pc;
			advice: [:context | context aroundAdvice];
			type: #around.
	adv2 :=  PhAdvice new
			pointcut: pc;
			advice: [:context | context aroundAdvice];
			type: #around.
	adv3 :=  PhAdvice new
			pointcut: pc;
			advice: [:context | context aroundAdvice];
			type: #around.
	
	asp := PhAspect new
		add: adv1;
		add: adv2;
		add: adv3.
	[asp install.
	
	self assert: (PhMockClass new foo includes: adv1).
	self assert: (PhMockClass new foo includes: adv2).
	self assert: (PhMockClass new foo includes: adv3).
	self assert: (PhMockClass new foo size = 3)] ensure: [asp uninstall.]! !

!PhPointcutTests methodsFor: 'context exposure' stamp: 'dgg 5/2/2011 15:48'!
testBeforeAdvicesExposure
	| asp adv1 adv2 adv3 pc |
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'foo';
		context: #(#advice).
	
	adv1 := PhAdvice new
		pointcut: pc;
		advice: [:context | 'foo'];
		type: #before.
	adv2 := PhAdvice new
		pointcut: pc;
		advice: [:context | 'bar'];
		type: #before.
	adv3 := PhAdvice new
		pointcut: pc;
		advice: [:context | 'foobar'];
		type: #before.    
	
	asp := PhAspect new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | context beforeAdvice];
			type: #around);
		add: adv1;
		add: adv2;
		add: adv3.
	[asp install.
	
	self assert: (PhMockClass new foo includes: adv1).
	self assert: (PhMockClass new foo includes: adv2).
	self assert: (PhMockClass new foo includes: adv3).
	self assert: (PhMockClass new foo size = 3)] ensure: [asp uninstall.]! !

!PhPointcutTests methodsFor: 'matching' stamp: 'DanielGaldames 3/21/2011 15:10'!
testClassNameMatch
	| pc targetSet |
	pc := PhPointcut new
		receivers: 'Morph';
		selectors: 'mouseEnter:'.
	targetSet := Set new.
	targetSet add: (Array with: Morph with: #mouseEnter:).
	self assert: (pc getPointcuts = targetSet).! !

!PhPointcutTests methodsFor: 'matching' stamp: 'DanielGaldames 3/21/2011 15:16'!
testClassNamePlusMatch
	| pc targetSet |
	pc := PhPointcut new
		receivers: 'Morph+';
		selectors: 'mouseEnter:'.
	targetSet := Set new.
	targetSet add: (Array with: Morph with: #mouseEnter:).
	Morph allSubclasses do: [:each |
		(each allSelectors anySatisfy: [:s | s = #mouseEnter:]) ifTrue: [
			targetSet add: (Array with: each with: #mouseEnter:)]].
	self assert: (pc getPointcuts = targetSet).
	! !

!PhPointcutTests methodsFor: 'matching' stamp: 'DanielGaldames 3/21/2011 19:59'!
testClassNameStarMatch
	| pc targetSet classCol regex |
	pc := PhPointcut new
		receivers: 'Morph*';
		selectors: 'mouseEnter:'.
	regex := 'Morph\w*' asRegex.
	classCol := (SmalltalkImage current allClasses) select: [:each | regex matches: each name].
	targetSet := Set new.
	targetSet add: (Array with: Morph with: #mouseEnter:).
	classCol do: [:each |
		(each allSelectors anySatisfy: [:s | s = #mouseEnter:]) ifTrue: [
			targetSet add: (Array with: each with: #mouseEnter:)]].
	self assert: (pc getPointcuts = targetSet).
	! !

!PhPointcutTests methodsFor: 'matching' stamp: 'DanielGaldames 3/21/2011 20:00'!
testClassNameStarPlusMatch
	| pc targetSet classCol regex |
	pc := PhPointcut new
		receivers: 'Morph*+';
		selectors: 'mouseEnter:'.
	regex := 'Morph\w*' asRegex.
	classCol := (SmalltalkImage current allClasses) select: [:each | regex matches: each name].
	targetSet := Set new.
	targetSet add: (Array with: Morph with: #mouseEnter:).
	classCol do: [:each |
		(each allSelectors anySatisfy: [:s | s = #mouseEnter:]) ifTrue: [
			targetSet add: (Array with: each with: #mouseEnter:)]].
	classCol do: [:each | each allSubclasses do: [:s|
			(s allSelectors anySatisfy: [:sel | sel = #mouseEnter:]) ifTrue: [
				targetSet add: (Array with: s with: #mouseEnter:)]]].
	self assert: (pc getPointcuts = targetSet).
	! !

!PhPointcutTests methodsFor: 'matching' stamp: 'DanielGaldames 3/21/2011 15:24'!
testClassStarNameMatch
	| pc targetSet classCol regex |
	pc := PhPointcut new
		receivers: '*Morph';
		selectors: 'mouseEnter:'.
	regex := '\w*Morph' asRegex.
	classCol := (SmalltalkImage current allClasses) select: [:each | regex matches: each name].
	targetSet := Set new.
	targetSet add: (Array with: Morph with: #mouseEnter:).
	classCol do: [:each |
		(each allSelectors anySatisfy: [:s | s = #mouseEnter:]) ifTrue: [
			targetSet add: (Array with: each with: #mouseEnter:)]].
	self assert: (pc getPointcuts = targetSet).
	! !

!PhPointcutTests methodsFor: 'matching' stamp: 'DanielGaldames 3/21/2011 20:00'!
testClassStarNamePlusMatch
	| pc targetSet classCol regex |
	pc := PhPointcut new
		receivers: '*Morph+';
		selectors: 'mouseEnter:'.
	regex := '\w*Morph' asRegex.
	classCol := (SmalltalkImage current allClasses) select: [:each | regex matches: each name].
	targetSet := Set new.
	targetSet add: (Array with: Morph with: #mouseEnter:).
	classCol do: [:each |
		(each allSelectors anySatisfy: [:s | s = #mouseEnter:]) ifTrue: [
			targetSet add: (Array with: each with: #mouseEnter:)]].
	classCol do: [:each | each allSubclasses do: [:s|
			(s allSelectors anySatisfy: [:sel | sel = #mouseEnter:]) ifTrue: [
				targetSet add: (Array with: s with: #mouseEnter:)]]].
	self assert: (pc getPointcuts = targetSet).
	! !

!PhPointcutTests methodsFor: 'matching' stamp: 'DanielGaldames 3/21/2011 15:25'!
testClassStarNameStarMatch
	| pc targetSet classCol regex |
	pc := PhPointcut new
		receivers: '*Morph*';
		selectors: 'mouseEnter:'.
	regex := '\w*Morph\w*' asRegex.
	classCol := (SmalltalkImage current allClasses) select: [:each | regex matches: each name].
	targetSet := Set new.
	targetSet add: (Array with: Morph with: #mouseEnter:).
	classCol do: [:each |
		(each allSelectors anySatisfy: [:s | s = #mouseEnter:]) ifTrue: [
			targetSet add: (Array with: each with: #mouseEnter:)]].
	self assert: (pc getPointcuts = targetSet).
	! !

!PhPointcutTests methodsFor: 'matching' stamp: 'DanielGaldames 3/21/2011 15:30'!
testClassStarNameStarPlusMatch
	| pc targetSet classCol regex |
	pc := PhPointcut new
		receivers: '*Morph*+';
		selectors: 'mouseEnter:'.
	regex := '\w*Morph\w*' asRegex.
	classCol := (SmalltalkImage current allClasses) select: [:each | regex matches: each name].
	targetSet := Set new.
	targetSet add: (Array with: Morph with: #mouseEnter:).
	classCol do: [:each |
		(each allSelectors anySatisfy: [:s | s = #mouseEnter:]) ifTrue: [
			targetSet add: (Array with: each with: #mouseEnter:)]].
	classCol do: [:each | each allSubclasses do: [:s|
			(s allSelectors anySatisfy: [:sel | sel = #mouseEnter:]) ifTrue: [
				targetSet add: (Array with: s with: #mouseEnter:)]]].
	self assert: (pc getPointcuts = targetSet).
	! !

!PhPointcutTests methodsFor: 'context exposure' stamp: 'dgg 5/2/2011 15:49'!
testCurrentAdvicesExposure
	| asp adv1 adv2 adv3 pc |
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'foo';
		context: #(#advice).
	
	adv1 := PhAdvice new
			pointcut: pc;
			advice: [:context | context currentAdvice];
			type: #around.
	adv2 :=  PhAdvice new
			pointcut: pc;
			advice: [:context | context currentAdvice];
			type: #around.
	adv3 :=  PhAdvice new
			pointcut: pc;
			advice: [:context | context currentAdvice];
			type: #around.
	
	asp := PhAspect new
		add: adv1;
		add: adv2;
		add: adv3.
	[asp install.
	
	self assert: (PhMockClass new foo includes: adv1).
	self assert: (PhMockClass new foo includes: adv2).
	self assert: (PhMockClass new foo includes: adv3).
	self assert: (PhMockClass new foo size = 3)] ensure: [asp uninstall.]! !

!PhPointcutTests methodsFor: 'custom-parser-match' stamp: 'dgg 6/9/2011 13:25'!
testCustomParserMatch
	| parser set pc |
	parser := (#any asParser plusGreedy: 'instanceVariableNames:' asParser) , 'instanceVariableNames:' asParser
		, (#any asParser plusGreedy: 'counter' asParser) , (#any asParser plusGreedy: 'classVariableNames:' asParser).
	pc := (PhPointcut new)
		receivers: parser;
		localSelectors: #any asParser;
		restrictToPackages: #('Phantom-Tests').
	set := Set new.
	(PackageOrganizer default packageNamed: 'Phantom-Tests' ifAbsent: nil) classes
		do: [ :class | 
			(class instVarNames includes: 'counter')
				ifTrue: [ class localSelectors do: [ :sel | set add: (Array with: class with: sel) ] ] ].
	self assert: pc getPointcuts = set! !

!PhPointcutTests methodsFor: 'context exposure' stamp: 'dgg 3/24/2012 12:11'!
testDynamicContext
	| pc asp |
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'test';
		context: #(#arguments #receiver);
		dynamicContextBlock: [:contextObject :thereContext |
			| idx |
			idx := (1 to: (thereContext contextStack size)) findLast: [:index |
				((thereContext contextStack at: index) receiver) == ((thereContext contextStack at: 1) receiver)].
			contextObject dynamicContext at: #sender put: (thereContext contextStack at: (idx+5)) receiver].  

	asp := PhAspect new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | context receiver test: (context dynamicContext at: #sender)];
			type: #before).
	[asp install.
	
	self assert: ((PhMockClass new test) == self)] ensure: [asp uninstall].    
		! !

!PhPointcutTests methodsFor: 'matching' stamp: 'dgg 9/29/2011 11:46'!
testGetMatchedClasses
	| pc |
	pc := PhPointcut new
		receivers: 'PhMockAspectFoo+';
		selectors: #any asParser.
	self assert: (pc getMatchedClasses asSet = PhMockAspectFoo withAllSubclasses asSet).! !

!PhPointcutTests methodsFor: 'matching' stamp: 'DanielGaldames 3/21/2011 23:05'!
testMetaClassNameMatch
	| pc targetSet |
	pc := PhPointcut new
		receivers: 'Morph class';
		selectors: 'newBounds:'.
	targetSet := Set new.
	targetSet add: (Array with: (Morph class) with: #newBounds:).
	self assert: (pc getPointcuts = targetSet).! !

!PhPointcutTests methodsFor: 'matching' stamp: 'DanielGaldames 3/21/2011 23:06'!
testMetaClassNamePlusMatch
	| pc targetSet |
	pc := PhPointcut new
		receivers: 'Morph class+';
		selectors: 'newBounds:'.
	targetSet := Set new.
	targetSet add: (Array with: (Morph class) with: #newBounds:).
	Morph class allSubclasses do: [:each |
		(each allSelectors anySatisfy: [:s | s = #newBounds:]) ifTrue: [
			targetSet add: (Array with: each with: #newBounds:)]].
	self assert: (pc getPointcuts = targetSet).
	! !

!PhPointcutTests methodsFor: 'matching' stamp: 'DanielGaldames 3/21/2011 23:11'!
testMetaClassNameStarMatch
	| pc targetSet classCol regex |
	pc := PhPointcut new
		receivers: 'Morph* class';
		selectors: 'newBounds:'.
	regex := 'Morph\w* class' asRegex.
	classCol := (SmalltalkImage current allClasses) collect: [:each | each class] thenSelect: [:each | regex matches: each name].
	targetSet := Set new.
	targetSet add: (Array with: (Morph class) with: #newBounds:).
	classCol do: [:each |
		(each allSelectors anySatisfy: [:s | s = #newBounds:]) ifTrue: [
			targetSet add: (Array with: each with: #newBounds:)]].
	self assert: (pc getPointcuts = targetSet).
	! !

!PhPointcutTests methodsFor: 'matching' stamp: 'DanielGaldames 3/21/2011 23:14'!
testMetaClassNameStarPlusMatch
	| pc targetSet classCol regex |
	pc := PhPointcut new
		receivers: 'Morph* class+';
		selectors: 'newBounds:'.
	regex := 'Morph\w* class' asRegex.
	classCol := (SmalltalkImage current allClasses) collect: [:each | each class] thenSelect: [:each | regex matches: each name].
	targetSet := Set new.
	targetSet add: (Array with: (Morph class) with: #newBounds:).
	classCol do: [:each |
		(each allSelectors anySatisfy: [:s | s = #newBounds:]) ifTrue: [
			targetSet add: (Array with: each with: #newBounds:)]].
	classCol do: [:each | each allSubclasses do: [:s|
			(s allSelectors anySatisfy: [:sel | sel = #newBounds:]) ifTrue: [
				targetSet add: (Array with: s with: #newBounds:)]]].
	self assert: (pc getPointcuts = targetSet).
	! !

!PhPointcutTests methodsFor: 'matching' stamp: 'DanielGaldames 3/21/2011 23:16'!
testMetaClassStarNameMatch
	| pc targetSet classCol regex |
	pc := PhPointcut new
		receivers: '*Morph class';
		selectors: 'newBounds:'.
	regex := '\w*Morph class' asRegex.
	classCol := (SmalltalkImage current allClasses) collect: [:each | each class] thenSelect: [:each | regex matches: each name].
	targetSet := Set new.
	targetSet add: (Array with: (Morph class) with: #newBounds:).
	classCol do: [:each |
		(each allSelectors anySatisfy: [:s | s = #newBounds:]) ifTrue: [
			targetSet add: (Array with: each with: #newBounds:)]].
	self assert: (pc getPointcuts = targetSet).
	! !

!PhPointcutTests methodsFor: 'matching' stamp: 'DanielGaldames 3/21/2011 23:18'!
testMetaClassStarNamePlusMatch
	| pc targetSet classCol regex |
	pc := PhPointcut new
		receivers: '*Morph class+';
		selectors: 'newBounds:'.
	regex := '\w*Morph class' asRegex.
	classCol := (SmalltalkImage current allClasses) collect: [:each | each class] thenSelect: [:each | regex matches: each name].
	targetSet := Set new.
	targetSet add: (Array with: (Morph class) with: #newBounds:).
	classCol do: [:each |
		(each allSelectors anySatisfy: [:s | s = #newBounds:]) ifTrue: [
			targetSet add: (Array with: each with: #newBounds:)]].
	classCol do: [:each | each allSubclasses do: [:s|
			(s allSelectors anySatisfy: [:sel | sel = #newBounds:]) ifTrue: [
				targetSet add: (Array with: s with: #newBounds:)]]].
	self assert: (pc getPointcuts = targetSet).
	! !

!PhPointcutTests methodsFor: 'matching' stamp: 'DanielGaldames 3/21/2011 23:20'!
testMetaClassStarNameStarMatch
	| pc targetSet classCol regex |
	pc := PhPointcut new
		receivers: '*Morph* class';
		selectors: 'newBounds:'.
	regex := '\w*Morph\w* class' asRegex.
	classCol := (SmalltalkImage current allClasses) collect: [:each | each class] thenSelect: [:each | regex matches: each name].
	targetSet := Set new.
	targetSet add: (Array with: (Morph class) with: #newBounds:).
	classCol do: [:each |
		(each allSelectors anySatisfy: [:s | s = #newBounds:]) ifTrue: [
			targetSet add: (Array with: each with: #newBounds:)]].
	self assert: (pc getPointcuts = targetSet).
	! !

!PhPointcutTests methodsFor: 'matching' stamp: 'DanielGaldames 3/21/2011 23:21'!
testMetaClassStarNameStarPlusMatch
	| pc targetSet classCol regex |
	pc := PhPointcut new
		receivers: '*Morph* class+';
		selectors: 'newBounds:'.
	regex := '\w*Morph\w* class' asRegex.
	classCol := (SmalltalkImage current allClasses) collect: [:each | each class ] thenSelect: [:each | regex matches: each name].
	targetSet := Set new.
	targetSet add: (Array with: (Morph class) with: #newBounds:).
	classCol do: [:each |
		(each allSelectors anySatisfy: [:s | s = #newBounds:]) ifTrue: [
			targetSet add: (Array with: each with: #newBounds:)]].
	classCol do: [:each | each allSubclasses do: [:s|
			(s allSelectors anySatisfy: [:sel | sel = #newBounds:]) ifTrue: [
				targetSet add: (Array with: s with: #newBounds:)]]].
	self assert: (pc getPointcuts = targetSet).
	! !

!PhPointcutTests methodsFor: 'context exposure' stamp: 'dgg 5/2/2011 15:50'!
testMethodExposure
	| pc asp |
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'test';
		context: #(#proceed).
	
	asp := PhAspect new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | context proceed];
			type: #around).
	[asp install.
	
	self assert: ((PhMockClass new test) = 'test')] ensure: [asp uninstall].    
		! !

!PhPointcutTests methodsFor: 'composing' stamp: 'dgg 3/25/2012 12:21'!
testNot
	| pc regex pcCol targetSet |
	pc := PhPointcut new
		receivers: '*Morph*';
		selectors: 'mouseEnter:'.
	regex := '\w*Morph\w*' asRegex.
	targetSet := Set new.
	(SmalltalkImage current allClasses) do: [:each | 
		(regex matches: each name asString) not
			ifTrue: [each allSelectors do: [:sel |
					targetSet add: (Array with: each with: sel)]] 
			ifFalse: [each allSelectors do: [:sel |
					(sel = #mouseEnter:) not
						ifTrue: [targetSet add: (Array with: each with: sel)]]].
		(regex matches: each class name asString) not
			ifTrue: [each class allSelectors do: [:sel |
					targetSet add: (Array with: (each class) with: sel)]] 
			ifFalse: [each class allSelectors do: [:sel |
					(sel = #mouseEnter:) not
						ifTrue: [targetSet add: (Array with: (each class) with: sel)]]]].
	pcCol := pc not getPointcuts.
	self assert: (pcCol = targetSet).! !

!PhPointcutTests methodsFor: 'composing' stamp: 'dgg 3/25/2012 12:21'!
testNot2
	| pc pc2 regex pcCol targetSet |
	pc := PhPointcut new
		receivers: '*Morph*';
		selectors: 'mouseEnter:'.
	
	pc2 := PhPointcut new
		receivers: (#word asParser star, ' class' asParser);
		selectors: (#any asParser star).
	     
	regex := '\w*Morph\w*' asRegex.
	targetSet := Set new.
	(SmalltalkImage current allClasses) do: [:each | 
		(regex matches: each name asString) not
			ifTrue: [each allSelectors do: [:sel |
					targetSet add: (Array with: each with: sel)]] 
			ifFalse: [each allSelectors do: [:sel |
					(sel = #mouseEnter:) not
						ifTrue: [targetSet add: (Array with: each with: sel)]]]].
	pcCol := ((pc not) & (pc2 not)) getPointcuts.
	self assert: (pcCol = targetSet).! !

!PhPointcutTests methodsFor: 'composing' stamp: 'dgg 3/25/2012 12:20'!
testNot3
	| pc1 pc2 col1 col2 |
	pc1 := PhPointcut new
		receivers: '*Morph*';
		selectors: 'mouseEnter:';
		restrictToPackages: #('Kernel' 'Morphic').  
	pc2 := PhPointcut new
		receivers: (#word asParser star, ' class' asParser);
		selectors: (#any asParser star);
		restrictToPackages: #('Kernel' 'Morphic').
		
	col1 := ((pc1 not) & (pc2 not)) getPointcuts.
	col2 := ( (pc1 | pc2) not ) getPointcuts.
	self assert: (col1 = col2)! !

!PhPointcutTests methodsFor: 'matching' stamp: 'dgg 6/9/2011 14:03'!
testOneArgumentKeywordSelectorMatch
	| set pc |
	pc := (PhPointcut new)
		receivers: 'PhPointcut';
		localSelectors: '_:'.
	set := Set new.
	PhPointcut localSelectors
		do: [ :sel | 
			(sel isKeyword and: [ (sel findBetweenSubStrs: ':') size = 1 ])
				ifTrue: [ set add: (Array with: PhPointcut with: sel) ] ].
	self deny: (set isEmpty).
	self assert: pc getPointcuts = set! !

!PhPointcutTests methodsFor: 'composing' stamp: 'dgg 9/18/2011 17:39'!
testOr
	| pc1 pc2 targetSet |
	pc1 := PhPointcut new
		receivers: 'Morph';
		selectors: 'mouseEnter:'.
	pc2 := PhPointcut new
		receivers: 'OrderedCollection';
		selectors: 'add:'.
	targetSet := Set new.
	targetSet
		add: (Array with: Morph with: #mouseEnter:);
		add: (Array with: OrderedCollection with: #add:).
	self assert: ( (pc1 | pc2) getPointcuts = targetSet )            ! !

!PhPointcutTests methodsFor: 'package-restriction' stamp: 'DanielGaldames 4/21/2011 00:55'!
testPackageRestriction
	| pc target |
	pc := PhPointcut new
		receivers: (#any asParser starGreedy: ('class:' asParser)), 'class:' asParser, #any asParser star;
		selectors: 'hash';
		restrictToPackages: #('Kernel').
	
	target := Set new.
	(PackageOrganizer default packageNamed: 'Kernel' ifAbsent: nil) ifNotNilDo: [:package |
		package classes do: [:class |
			(class isKindOf: Trait) ifFalse: [
				(class allSelectors includes: #hash) ifTrue: [
					target add: (Array with: class with: #hash)]]]].
	self assert: (target = pc getPointcuts)! !

!PhPointcutTests methodsFor: 'matching-arrays' stamp: 'DanielGaldames 3/21/2011 20:07'!
testReceiverArray
	| pc targetSet |
	pc := PhPointcut new
		receivers: (Array with: 'Morph' with: 'BorderedMorph');
		selectors: 'mouseEnter:'.
	targetSet := Set new.
	targetSet 
		add: (Array with: Morph with: #mouseEnter:);
		add: (Array with: BorderedMorph with: #mouseEnter:).
		
	self assert: (pc getPointcuts = targetSet)  
		! !

!PhPointcutTests methodsFor: 'context exposure' stamp: 'dgg 5/2/2011 15:50'!
testReceiverExposure
	| pc asp mock |
	mock := PhMockClass new.
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'test';
		context: #(#receiver).
	
	asp := PhAspect new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | context receiver];
			type: #around).
	[asp install.
	
	self assert: (mock test = mock)] ensure: [asp uninstall].    
		! !

!PhPointcutTests methodsFor: 'matching-arrays' stamp: 'DanielGaldames 3/21/2011 20:11'!
testReceiverSelectorArray
	| pc targetSet |
	pc := PhPointcut new
		receivers: (Array with: 'Morph' with: 'BorderedMorph');
		selectors: (Array with: 'mouseEnter:' with: 'mouseDown:').
	targetSet := Set new.
	targetSet 
		add: (Array with: Morph with: #mouseEnter:);
		add: (Array with: Morph with: #mouseDown:);
		add: (Array with: BorderedMorph with: #mouseEnter:);
		add: (Array with: BorderedMorph with: #mouseDown:).
		
	self assert: (pc getPointcuts = targetSet)  
		! !

!PhPointcutTests methodsFor: 'matching-arrays' stamp: 'DanielGaldames 3/21/2011 20:10'!
testSelectorArray
	| pc targetSet |
	pc := PhPointcut new
		receivers: 'Morph';
		selectors: (Array with: 'mouseEnter:' with: 'mouseDown:').
	targetSet := Set new.
	targetSet 
		add: (Array with: Morph with: #mouseEnter:);
		add: (Array with: Morph with: #mouseDown:).
		
	self assert: (pc getPointcuts = targetSet)  
		! !

!PhPointcutTests methodsFor: 'context exposure' stamp: 'dgg 5/2/2011 15:51'!
testSelectorExposure
	| pc asp |
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'test';
		context: #(#selector).
	
	asp := PhAspect new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | context selector];
			type: #around).
	[asp install.
	
	self assert: ((PhMockClass new test) = #test)] ensure: [asp uninstall].    
		! !

!PhPointcutTests methodsFor: 'context exposure' stamp: 'dgg 5/2/2011 15:51'!
testSenderExposure
	| pc asp |
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'test';
		context: #(#sender).
	
	asp := PhAspect new
		add: (PhAdvice new
			pointcut: pc;
			advice: [:context | context sender name asString];
			type: #around).
	[asp install.
	
	self assert: ((PhMockClass new test) = self name asString)] ensure: [asp uninstall].    
		! !

!PhPointcutTests methodsFor: 'context exposure' stamp: 'dgg 1/22/2012 12:16'!
testSenderExposureFromSendTo
	| pc asp mock |
	pc := PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'test';
		context: #(#sender).
	mock := PhMockClass new.
	asp := PhAspect new
		add: (PhAdvice
			around: pc
			send: #getSender: to: mock).
	[asp install.
	
	self assert: ((PhMockClass new test) = self)] ensure: [asp uninstall].    
		! !

!PhPointcutTests methodsFor: 'matching' stamp: 'dgg 6/9/2011 14:03'!
testTwoArgumentKeywordSelectorMatch
	| set pc |
	pc := (PhPointcut new)
		receivers: 'PhPointcut';
		localSelectors: '_:_:'.
	set := Set new.
	PhPointcut localSelectors
		do: [ :sel | 
			(sel isKeyword and: [ (sel findBetweenSubStrs: ':') size = 2 ])
				ifTrue: [ set add: (Array with: PhPointcut with: sel) ] ].
	self deny: (set isEmpty).
	self assert: pc getPointcuts = set! !

!PhPointcutTests methodsFor: 'matching' stamp: 'dgg 6/9/2011 14:11'!
testTwoArgumentKeywordSelectorWithStringMatch
	| set pc |
	pc := (PhPointcut new)
		receivers: 'LinkedList';
		localSelectors: 'add:_:'.
	set := Set new.
	LinkedList localSelectors
		do: [ :sel | 
			((sel isKeyword) and: [((sel findBetweenSubStrs: ':') size = 2) and: [(((sel findBetweenSubStrs: ':') at: 1) = 'add')]])
				ifTrue: [ set add: (Array with: LinkedList with: sel) ] ].
	self deny: (set isEmpty).
	self assert: pc getPointcuts = set! !

!PhPointcutTests methodsFor: 'matching' stamp: 'dgg 6/9/2011 14:02'!
testUnarySelectorMatch
	| set pc |
	pc := (PhPointcut new)
		receivers: 'PhPointcut';
		localSelectors: '_'.
	set := Set new.
	PhPointcut localSelectors
		do: [ :sel | 
			sel isUnary
				ifTrue: [ set add: (Array with: PhPointcut with: sel) ] ].
	self deny: (set isEmpty).
	self assert: pc getPointcuts = set! !

TestCase subclass: #PhReceiverParserTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

!PhReceiverParserTests methodsFor: 'matchTests' stamp: 'DanielGaldames 3/16/2011 00:14'!
testClassMatch
	| targetCol generatedCol parser |
	targetCol := Set new.
	targetCol add: Morph.
	generatedCol := Set new.
	parser := (PhReceiverParser new) parse: 'Morph'.
	SmalltalkImage current allClasses do: [:each |
		(parser matches: each definition) ifTrue: [generatedCol add: each].
		(parser matches: each class definition) ifTrue: [generatedCol add: each class]].
	self assert: (generatedCol = targetCol)! !

!PhReceiverParserTests methodsFor: 'matchMetaTests' stamp: 'DanielGaldames 3/16/2011 00:20'!
testClassMetaMatch
	| targetCol generatedCol parser |
	targetCol := Set new.
	targetCol add: Morph class.
	generatedCol := Set new.
	parser := (PhReceiverParser new) parse: 'Morph class'.
	SmalltalkImage current allClasses do: [:each |
		(parser matches: each definition) ifTrue: [generatedCol add: each].
		(parser matches: each class definition) ifTrue: [generatedCol add: each class]].
	self assert: (generatedCol = targetCol)! !

!PhReceiverParserTests methodsFor: 'matchMetaTests' stamp: 'DanielGaldames 3/21/2011 14:52'!
testClassMetaPlusMatch
	| targetCol generatedCol parser |
	targetCol := Set new.
	targetCol
		add: Morph class;
		addAll: (Morph class allSubclasses).  
	
	generatedCol := Set new.
	parser := (PhReceiverParser new) parse: 'Morph class+'.
	SmalltalkImage current allClasses do: [:each |
		(parser matches: each definition) ifTrue: [generatedCol add: each].
		(parser matches: each class definition) ifTrue: [generatedCol add: each class]].
	self assert: (generatedCol = targetCol)! !

!PhReceiverParserTests methodsFor: 'as yet unclassified' stamp: 'dgg 1/14/2012 13:36'!
testClassMetaStarMatch
	| targetCol generatedCol parser |
	targetCol := Set new.
	targetCol addAll: Class allSubclasses .
	generatedCol := Set new.
	parser := (PhReceiverParser new) parse: '* class'.
	SmalltalkImage current allClasses do: [:each |
		(parser matches: each definition) ifTrue: [generatedCol add: each].
		(parser matches: each class definition) ifTrue: [generatedCol add: each class]].
	self assert: (generatedCol = targetCol)! !

!PhReceiverParserTests methodsFor: 'parserTests' stamp: 'DanielGaldames 3/15/2011 23:17'!
testClassName
	| parser |
	parser := PhReceiverParser new.
	self assert: (parser matches: 'Morph').
	self assert: (parser matches: 'Morph class').
	self deny: (parser matches: '1Morph').
	self deny: (parser matches: ' Morph').
	self deny: (parser matches: 'Mor ph').
	self deny: (parser matches: 'Morph foo').
	! !

!PhReceiverParserTests methodsFor: 'parserTests' stamp: 'DanielGaldames 3/15/2011 23:26'!
testClassNameMetaPlus
	| parser |
	parser := PhReceiverParser new.
	self assert: (parser matches: 'Morph class+').
	self assert: (parser matches: '*Morph class+').
	self assert: (parser matches: '*Morph* class+').
	self assert: (parser matches: 'Morph* class+').
	self deny: (parser matches: '*Morph+ class').
	self deny: (parser matches: '*Morph+* class').
	self deny: (parser matches: 'Morph*+ class').! !

!PhReceiverParserTests methodsFor: 'parserTests' stamp: 'DanielGaldames 3/15/2011 23:25'!
testClassNameMetaStar
	| parser |
	parser := PhReceiverParser new.
	self assert: (parser matches: '*Morph class').
	self assert: (parser matches: '*Morph* class').
	self assert: (parser matches: 'Morph* class').
	self deny: (parser matches: '*Morph class*').
	self deny: (parser matches: 'Morph class*').

	! !

!PhReceiverParserTests methodsFor: 'parserTests' stamp: 'DanielGaldames 3/15/2011 23:27'!
testClassNamePlus
	| parser |
	parser := PhReceiverParser new.
	self assert: (parser matches: 'Morph+').
	self assert: (parser matches: '*Morph*+').
	self assert: (parser matches: 'Morph*+').
	self assert: (parser matches: '*Morph+').
	self deny: (parser matches: '*Morph +').
	self deny: (parser matches: '*Morph+*').
	self deny: (parser matches: 'Morph+*').! !

!PhReceiverParserTests methodsFor: 'parserTests' stamp: 'DanielGaldames 3/30/2011 20:32'!
testClassNameStar
	| parser |
	parser := PhReceiverParser new.
	self assert: (parser matches: '*Morph').
	self assert: (parser matches: '*Morph*').
	self assert: (parser matches: 'Morph*').
	self deny: (parser matches: '*Morph *').
	self deny: (parser matches: '* Morph *').
	! !

!PhReceiverParserTests methodsFor: 'matchTests' stamp: 'DanielGaldames 3/21/2011 14:51'!
testClassPlusMatch
	| targetCol generatedCol parser |
	targetCol := Set new.
	targetCol
		add: Morph;
		addAll: (Morph allSubclasses).  
	
	generatedCol := Set new.
	parser := (PhReceiverParser new) parse: 'Morph+'.
	SmalltalkImage current allClasses do: [:each |
		(parser matches: each definition) ifTrue: [generatedCol add: each].
		(parser matches: each class definition) ifTrue: [generatedCol add: each class]].
	self assert: (generatedCol = targetCol)! !

!PhReceiverParserTests methodsFor: 'matchTests' stamp: 'DanielGaldames 3/16/2011 00:54'!
testClassStarMatch
	| targetCol generatedCol parser regex |
	regex := 'Morph\w*' asRegex.
	targetCol := Set new.
	generatedCol := Set new.
	parser := (PhReceiverParser new) parse: 'Morph*'.
	SmalltalkImage current allClasses do: [:each |
		(parser matches: each definition) ifTrue: [generatedCol add: each].
		(regex matches: each name asString) ifTrue: [targetCol add: each].
		(parser matches: each class definition) ifTrue: [generatedCol add: each class]].
	self assert: (generatedCol = targetCol)! !

!PhReceiverParserTests methodsFor: 'matchMetaTests' stamp: 'DanielGaldames 3/18/2011 01:12'!
testClassStarMetaMatch
	| targetCol generatedCol parser regex |
	targetCol := Set new.
	targetCol add: Morph class.
	generatedCol := Set new.
	parser := (PhReceiverParser new) parse: 'Morph* class'.
	regex := 'Morph\w* class' asRegex.
	SmalltalkImage current allClasses do: [:each |
		(parser matches: each definition) ifTrue: [generatedCol add: each].
		(parser matches: each class definition) ifTrue: [generatedCol add: each class].
		(regex matches: each class name asString) ifTrue: [targetCol add: each class]].
	self assert: (generatedCol = targetCol)! !

!PhReceiverParserTests methodsFor: 'matchMetaTests' stamp: 'DanielGaldames 3/21/2011 14:52'!
testClassStarMetaPlusMatch
	| targetCol generatedCol parser regex tmp |
	targetCol := Set new.
	targetCol add: Morph class.
	generatedCol := Set new.
	tmp := Set new.
	parser := (PhReceiverParser new) parse: 'Morph* class+'.
	regex := 'Morph\w* class' asRegex.
	SmalltalkImage current allClasses do: [:each |
		(parser matches: each definition) ifTrue: [generatedCol add: each].
		(parser matches: each class definition) ifTrue: [generatedCol add: each class].
		(regex matches: each class name asString) ifTrue: [targetCol add: each class]].
	targetCol do: [:each | tmp addAll: each allSubclasses].
	targetCol addAll: tmp.  
	self assert: (generatedCol = targetCol)! !

!PhReceiverParserTests methodsFor: 'matchTests' stamp: 'DanielGaldames 3/21/2011 14:52'!
testClassStarPlusMatch
	| targetCol generatedCol parser regex tmp |
	regex := 'Morph\w*' asRegex.
	targetCol := Set new.
	generatedCol := Set new.
	tmp := Set new.
	parser := (PhReceiverParser new) parse: 'Morph*+'.
	SmalltalkImage current allClasses do: [:each |
		(parser matches: each definition) ifTrue: [generatedCol add: each].
		(regex matches: each name asString) ifTrue: [targetCol add: each].
		(parser matches: each class definition) ifTrue: [generatedCol add: each class]].
	targetCol do: [:each | tmp addAll: each allSubclasses ].
	targetCol addAll: tmp.  
	self assert: (generatedCol = targetCol)! !

!PhReceiverParserTests methodsFor: 'matchTests' stamp: 'DanielGaldames 3/16/2011 00:54'!
testStarClassMatch
	| targetCol generatedCol parser regex |
	regex := '\w*Morph' asRegex.
	targetCol := Set new.
	generatedCol := Set new.
	parser := (PhReceiverParser new) parse: '*Morph'.
	SmalltalkImage current allClasses do: [:each |
		(parser matches: each definition) ifTrue: [generatedCol add: each].
		(regex matches: each name asString) ifTrue: [targetCol add: each].
		(parser matches: each class definition) ifTrue: [generatedCol add: each class]].
	self assert: (generatedCol = targetCol)! !

!PhReceiverParserTests methodsFor: 'matchMetaTests' stamp: 'DanielGaldames 3/18/2011 01:13'!
testStarClassMetaMatch
	| targetCol generatedCol parser regex |
	targetCol := Set new.
	targetCol add: Morph class.
	generatedCol := Set new.
	parser := (PhReceiverParser new) parse: '*Morph class'.
	regex := '\w*Morph class' asRegex.
	SmalltalkImage current allClasses do: [:each |
		(parser matches: each definition) ifTrue: [generatedCol add: each].
		(parser matches: each class definition) ifTrue: [generatedCol add: each class].
		(regex matches: each class name asString) ifTrue: [targetCol add: each class]].
	self assert: (generatedCol = targetCol)! !

!PhReceiverParserTests methodsFor: 'matchMetaTests' stamp: 'DanielGaldames 3/21/2011 14:53'!
testStarClassMetaPlusMatch
	| targetCol generatedCol parser regex tmp |
	targetCol := Set new.
	targetCol add: Morph class.
	generatedCol := Set new.
	tmp := Set new.
	parser := (PhReceiverParser new) parse: '*Morph class+'.
	regex := '\w*Morph class' asRegex.
	SmalltalkImage current allClasses do: [:each |
		(parser matches: each definition) ifTrue: [generatedCol add: each].
		(parser matches: each class definition) ifTrue: [generatedCol add: each class].
		(regex matches: each class name asString) ifTrue: [targetCol add: each class]].
	targetCol do: [:each | tmp addAll: each allSubclasses].
	targetCol addAll: tmp.
	self assert: (generatedCol = targetCol)! !

!PhReceiverParserTests methodsFor: 'matchTests' stamp: 'DanielGaldames 3/21/2011 14:52'!
testStarClassPlusMatch
	| targetCol generatedCol parser regex tmp |
	regex := '\w*Morph' asRegex.
	targetCol := Set new.
	generatedCol := Set new.
	tmp := Set new.
	parser := (PhReceiverParser new) parse: '*Morph+'.
	SmalltalkImage current allClasses do: [:each |
		(parser matches: each definition) ifTrue: [generatedCol add: each].
		(regex matches: each name asString) ifTrue: [targetCol add: each].
		(parser matches: each class definition) ifTrue: [generatedCol add: each class]].
	targetCol do: [:each | tmp addAll: each allSubclasses ].
	targetCol addAll: tmp.  
	self assert: (generatedCol = targetCol)! !

!PhReceiverParserTests methodsFor: 'matchTests' stamp: 'DanielGaldames 3/16/2011 00:53'!
testStarClassStarMatch
	| targetCol generatedCol parser regex |
	regex := '\w*Morph\w*' asRegex.
	targetCol := Set new.
	generatedCol := Set new.
	parser := (PhReceiverParser new) parse: '*Morph*'.
	SmalltalkImage current allClasses do: [:each |
		(parser matches: each definition) ifTrue: [generatedCol add: each].
		(regex matches: each name asString) ifTrue: [targetCol add: each].
		(parser matches: each class definition) ifTrue: [generatedCol add: each class]].
	self assert: (generatedCol = targetCol)! !

!PhReceiverParserTests methodsFor: 'matchMetaTests' stamp: 'DanielGaldames 3/18/2011 01:13'!
testStarClassStarMetaMatch
	| targetCol generatedCol parser regex |
	targetCol := Set new.
	targetCol add: Morph class.
	generatedCol := Set new.
	parser := (PhReceiverParser new) parse: '*Morph* class'.
	regex := '\w*Morph\w* class' asRegex.
	SmalltalkImage current allClasses do: [:each |
		(parser matches: each definition) ifTrue: [generatedCol add: each].
		(parser matches: each class definition) ifTrue: [generatedCol add: each class].
		(regex matches: each class name asString) ifTrue: [targetCol add: each class]].
	self assert: (generatedCol = targetCol)! !

!PhReceiverParserTests methodsFor: 'matchMetaTests' stamp: 'DanielGaldames 3/21/2011 14:53'!
testStarClassStarMetaPlusMatch
	| targetCol generatedCol parser regex tmp |
	targetCol := Set new.
	targetCol add: Morph class.
	generatedCol := Set new.
	tmp := Set new.
	parser := (PhReceiverParser new) parse: '*Morph* class+'.
	regex := '\w*Morph\w* class' asRegex.
	SmalltalkImage current allClasses do: [:each |
		(parser matches: each definition) ifTrue: [generatedCol add: each].
		(parser matches: each class definition) ifTrue: [generatedCol add: each class].
		(regex matches: each class name asString) ifTrue: [targetCol add: each class]].
	targetCol do: [:each | tmp addAll: each allSubclasses ].
	targetCol addAll: tmp.  
	self assert: (generatedCol = targetCol)! !

!PhReceiverParserTests methodsFor: 'matchTests' stamp: 'DanielGaldames 3/21/2011 14:52'!
testStarClassStarPlusMatch
	| targetCol generatedCol parser regex tmp |
	regex := '\w*Morph\w*' asRegex.
	targetCol := Set new.
	generatedCol := Set new.
	tmp := Set new.
	parser := (PhReceiverParser new) parse: '*Morph*+'.
	SmalltalkImage current allClasses do: [:each |
		(parser matches: each definition) ifTrue: [generatedCol add: each].
		(regex matches: each name asString) ifTrue: [targetCol add: each].
		(parser matches: each class definition) ifTrue: [generatedCol add: each class]].
	targetCol do: [:each | tmp addAll: each allSubclasses ].
	targetCol addAll: tmp.    
	self assert: (generatedCol = targetCol)! !

TestCase subclass: #PhSelectorParserTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

!PhSelectorParserTests methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 3/18/2011 19:14'!
testAnyMatch
	| parser |
	parser := (PhSelectorParser new) parse: '_'.
	self assert: (parser matches: 'bar').
	self assert: (parser matches: 'foobar').
	self deny: (parser matches: 'foobar:').
	self deny: (parser matches: 'foo:bla:hh:bar:').
	self deny: (parser matches: 'foo:bar:').
	self deny: (parser matches: 'bla:foo:hh:bar:').
	self deny: (parser matches: 'bar:').
	
	  
	! !

!PhSelectorParserTests methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 3/18/2011 19:12'!
testAnyMultiwordMatch
	| parser |
	parser := (PhSelectorParser new) parse: '_:bar:'.
	self assert: (parser matches: 'any:bar:').
	self assert: (parser matches: 'foo:bar:').
	self deny: (parser matches: 'foo:bla:hh:bar:').
	self deny: (parser matches: 'bla:foo:hh:bar:').
	self deny: (parser matches: 'bar:').
	
	  
	! !

!PhSelectorParserTests methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 3/18/2011 19:13'!
testAnyWithArgsMatch
	| parser |
	parser := (PhSelectorParser new) parse: '_:'.
	self assert: (parser matches: 'bar:').
	self assert: (parser matches: 'foobar:').
	self deny: (parser matches: 'foobar').
	self deny: (parser matches: 'foo:bla:hh:bar:').
	self deny: (parser matches: 'foo:bar:').
	self deny: (parser matches: 'bla:foo:hh:bar:').
	
	  
	! !

!PhSelectorParserTests methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 3/18/2011 17:57'!
testBinary
	| parser |
	parser := PhSelectorParser new.
	self assert: (parser matches: '*').
	self assert: (parser matches: '***').
	self assert: (parser matches: '==>').
	self assert: (parser matches: '/').
	self deny: (parser matches: '_/').
	self deny: (parser matches: '_/:').
	self deny: (parser matches: '+:').
	self deny: (parser matches: '/_').! !

!PhSelectorParserTests methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 3/18/2011 18:06'!
testBinaryMatch
	| parser |
	parser := (PhSelectorParser new) parse: '*'.
	self assert: (parser matches: '*').
	self deny: (parser matches: '**').
	self deny: (parser matches: '***').
	self deny: (parser matches: '=>').
	self deny: (parser matches: 'foo').
	self deny: (parser matches: 'foo:').
	
	
	! !

!PhSelectorParserTests methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 3/18/2011 17:57'!
testMultiword
	| parser |
	parser := PhSelectorParser new.
	self assert: (parser matches: 'foo:').
	self assert: (parser matches: 'foo:bar:').
	self assert: (parser matches: 'foo:bar:foobar:').
	self deny: (parser matches: '1foo:').
	self deny: (parser matches: 'foo:bar').
	self deny: (parser matches: 'foo:+bar:').
	! !

!PhSelectorParserTests methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 3/18/2011 19:12'!
testMultiwordAny
	| parser |
	parser := PhSelectorParser new.
	self assert: (parser matches: '_:foo:').
	self assert: (parser matches: '_').
	self assert: (parser matches: '_:').
	self assert: (parser matches: 'foo:_:').
	self assert: (parser matches: '_:foo:_:_:bar:').
	self deny: (parser matches: '_foo:_:bar:_:').
	self deny: (parser matches: '_:foo').
	self deny: (parser matches: 'foo_:').
	self deny: (parser matches: '_foo').
	self deny: (parser matches: 'foo:_:bar').! !

!PhSelectorParserTests methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 3/18/2011 18:46'!
testMultiwordAnyMatch
	| parser |
	parser := (PhSelectorParser new) parse: 'bar:_:'.
	self assert: (parser matches: 'bar:foo:').
	self assert: (parser matches: 'bar:bla:').
	self deny: (parser matches: 'foo:bla:hh:bar:').
	self deny: (parser matches: 'foo:bar:').
	self deny: (parser matches: 'bla:foo:hh:bar:').
	self deny: (parser matches: 'bar:').
	
	  
	! !

!PhSelectorParserTests methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 3/18/2011 18:46'!
testMultiwordAnyMultiwordMatch
	| parser |
	parser := (PhSelectorParser new) parse: 'foo:_:bar:'.
	self assert: (parser matches: 'foo:any:bar:').
	self assert: (parser matches: 'foo:bla:bar:').
	self deny: (parser matches: 'foo:bla:hh:bar:').
	self deny: (parser matches: 'foo:bar:').
	self deny: (parser matches: 'bla:foo:hh:bar:').
	
	  
	! !

!PhSelectorParserTests methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 3/18/2011 18:09'!
testMultiwordMatch
	| parser |
	parser := (PhSelectorParser new) parse: 'foo:'.
	self assert: (parser matches: 'foo:').
	self deny: (parser matches: 'foo').
	self deny: (parser matches: 'foo:bar:').
	self deny: (parser matches: 'foo:bar:foobar:').
	! !

!PhSelectorParserTests methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 3/18/2011 18:46'!
testUnary
	| parser |
	parser := PhSelectorParser new.
	self assert: (parser matches: 'foo').
	self deny: (parser matches: '1foo').
	self deny: (parser matches: '_:foo').
	self deny: (parser matches: '#foo').
	self deny: (parser matches: '*foo').! !

!PhSelectorParserTests methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 3/30/2011 20:32'!
testUnaryMatch
	| parser |
	parser := (PhSelectorParser new) parse: 'foo'.
	self assert: (parser matches: 'foo').
	self deny: (parser matches: 'foobar').
	self deny: (parser matches: 'foo:').
	self deny: (parser matches: 'bar').
	self deny: (parser matches: '*').! !

PhAdvice subclass: #PhMockAdvice
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

!PhMockAdvice methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 3/27/2011 21:16'!
test
	^'test'! !

!PhMockAdvice methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 3/22/2011 00:44'!
test: aContext
	^ 'test'! !

PhAspect subclass: #PhMockAspect1
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

PhAspect subclass: #PhMockAspect2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

PhAspect subclass: #PhMockAspect3
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

PhAspect subclass: #PhMockAspect4
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

PhAspect subclass: #PhMockAspect5
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

PhAspect subclass: #PhMockAspect6
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

PhAspect subclass: #PhMockAspectFoo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

PhMockAspectFoo subclass: #PhMockAspectFoo1
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

PhMockAspectFoo subclass: #PhMockAspectFoo2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

PhMockAspectFoo2 subclass: #PhMockFooAspect4
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

PhMockAspectFoo2 subclass: #PhMockFooAspect5
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

PhMockAspectFoo subclass: #PhMockAspectFoo3
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

Object subclass: #PhMockClass
	instanceVariableNames: 'counter testString'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

!PhMockClass methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 3/27/2011 21:17'!
adv
	^ 'test'! !

!PhMockClass methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 3/22/2011 01:02'!
adv: aContext
	^ 'test'! !

!PhMockClass methodsFor: 'test methods' stamp: 'DanielGaldames 3/20/2011 22:41'!
bar
	Transcript show: ' is '.! !

!PhMockClass methodsFor: 'as yet unclassified' stamp: 'dgg 11/28/2011 20:26'!
callMethod2
	^ self method2! !

!PhMockClass methodsFor: 'accessing' stamp: 'DanielGaldames 3/21/2011 13:06'!
counter
	^counter! !

!PhMockClass methodsFor: 'accessing' stamp: 'johanfabry 10/7/2011 22:11'!
counter: anObject
	counter := anObject! !

!PhMockClass methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 3/22/2011 00:20'!
echo: aString
	^aString! !

!PhMockClass methodsFor: 'as yet unclassified' stamp: 'dgg 3/25/2012 12:18'!
foo! !

!PhMockClass methodsFor: 'as yet unclassified' stamp: 'dgg 1/22/2012 12:13'!
getSender: ctx
	^ ctx sender! !

!PhMockClass methodsFor: 'initialize-release' stamp: 'DanielGaldames 3/21/2011 13:19'!
initialize
	testString := 'test'.
	counter := 0.! !

!PhMockClass methodsFor: 'as yet unclassified' stamp: 'dgg 11/28/2011 20:26'!
method2
	 ^'mockBehavior' ! !

!PhMockClass methodsFor: 'test methods' stamp: 'DanielGaldames 3/21/2011 13:07'!
minus
	counter := counter -1! !

!PhMockClass methodsFor: 'test methods' stamp: 'DanielGaldames 3/21/2011 13:07'!
plus
	counter := counter +1! !

!PhMockClass methodsFor: 'test methods' stamp: 'DanielGaldames 3/21/2011 01:52'!
test
	^testString.! !

!PhMockClass methodsFor: 'test methods' stamp: 'DanielGaldames 3/25/2011 15:49'!
test: aString
	testString := aString.! !

!PhMockClass methodsFor: 'accessing' stamp: 'DanielGaldames 3/21/2011 12:37'!
testString
	^testString.! !

!PhMockClass methodsFor: 'as yet unclassified' stamp: 'dgg 10/9/2011 23:43'!
toCallTest
	^ self test! !

PhMockClass subclass: #PhMockSubclass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

Object subclass: #PhMockMockClass
	instanceVariableNames: 'counter testString'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!

!PhMockMockClass methodsFor: 'as yet unclassified' stamp: 'dgg 5/30/2011 01:27'!
callTestMock
	^ PhMockClass new test! !

Object subclass: #PhNoOneMock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Tests'!
